"""Brazil locale data — pt_BR"""

FIRST_NAMES_MALE = [
    "Miguel", "Arthur", "Bernardo", "Heitor", "Davi", "Lorenzo", "Théo", "Pedro", "Gabriel",
    "Enzo", "Matheus", "Lucas", "Rafael", "João", "Gustavo", "Felipe", "Guilherme",
    "Nicolas", "Samuel", "Daniel", "Henrique", "Eduardo", "Leonardo", "Bruno", "Caio",
    "Vinícius", "Diego", "André", "Rodrigo", "Thiago", "Fernando", "Marcelo", "Ricardo",
    "Roberto", "Alexandre", "Carlos", "Paulo", "Francisco", "José", "Antônio", "Marcos",
    "Leandro", "Fábio", "Sérgio", "Renato", "Márcio", "Cláudio", "Luís", "Rogério",
    "Jorge", "Adriano", "Flávio", "Julio", "Luciano", "Renan", "Igor", "Otávio", "Murilo",
    "Cauã", "Luan", "Breno", "Benício", "Vicente", "Emanuel", "Ravi", "Bryan", "Isaac",
    "Anthony", "Pietro", "Henry", "Gael", "Bento", "Noah", "Caleb", "Joaquim", "Raul",
    "Martin", "Ian", "Tomás", "Augusto", "César", "Elias", "Levi", "Nathan", "Yuri",
    "Pablo", "Tales", "Danilo", "Emerson", "Hugo", "Mateus", "Tiago", "Anderson",
    "Cristiano", "Edson", "Neymar", "Ronaldo", "Valdir", "Wanderley", "Wilson", "Ivan",
    "Nelson", "Osvaldo",
]

FIRST_NAMES_FEMALE = [
    "Alice", "Sophia", "Helena", "Valentina", "Laura", "Isabella", "Manuela", "Júlia",
    "Heloísa", "Luísa", "Maria", "Lorena", "Lívia", "Giovanna", "Beatriz", "Gabriela",
    "Letícia", "Isadora", "Mariana", "Ana", "Camila", "Rafaela", "Fernanda", "Carolina",
    "Juliana", "Amanda", "Bruna", "Larissa", "Tatiana", "Patrícia", "Vanessa", "Adriana",
    "Renata", "Luciana", "Simone", "Cristiane", "Cláudia", "Rosana", "Sandra", "Mônica",
    "Márcia", "Denise", "Sônia", "Vera", "Regina", "Célia", "Eliane", "Lúcia", "Tereza",
    "Francisca", "Rita", "Rosa", "Marta", "Débora", "Priscila", "Aline", "Natália",
    "Viviane", "Daniela", "Carla", "Bianca", "Marina", "Yasmin", "Lara", "Cecília",
    "Clara", "Emanuelly", "Eloá", "Antonella", "Maya", "Agatha", "Rebeca", "Sophie",
    "Alícia", "Luna", "Melissa", "Catarina", "Stella", "Jade", "Olívia", "Aurora",
    "Isis", "Esther", "Elisa", "Liz", "Pietra", "Milena", "Raquel", "Vitória",
    "Alessandra", "Flávia", "Thaís", "Jéssica", "Fabiana", "Michele", "Andréia",
    "Elisângela", "Joana", "Iara", "Ingrid", "Paloma",
]

LAST_NAMES = [
    "Silva", "Santos", "Oliveira", "Souza", "Rodrigues", "Ferreira", "Alves", "Pereira",
    "Lima", "Gomes", "Costa", "Ribeiro", "Martins", "Carvalho", "Araújo", "Melo",
    "Barbosa", "Rocha", "Cardoso", "Nascimento", "Moreira", "Nunes", "Marques", "Machado",
    "Mendes", "Freitas", "Monteiro", "Cavalcanti", "Dias", "Castro", "Campos", "Teixeira",
    "Moura", "Vieira", "Correia", "Pinto", "Batista", "Soares", "Reis", "Lopes",
    "Miranda", "Duarte", "Ramos", "Fernandes", "Barros", "Farias", "Bezerra", "Amaral",
    "Pires", "Cunha", "Sales", "Sampaio", "Azevedo", "Medeiros", "Siqueira", "Andrade",
    "Borges", "Fonseca", "Peixoto", "Nogueira", "Aguiar", "Brito", "Leal", "Rezende",
    "Coelho", "Xavier", "Alencar", "Lacerda", "Guimarães", "Tavares", "Pacheco", "Viana",
    "Queiroz", "Morais", "Paiva", "Bastos", "Carneiro", "Matos", "Santiago", "Magalhães",
    "Rangel", "Assis", "Dantas", "Silveira", "Braga", "Toledo", "Porto", "Dourado",
    "Figueiredo", "Coutinho", "Moraes", "Simões", "Prado", "Franco", "Barreto", "Vargas",
    "Trindade", "Valente", "Mesquita", "Sena",
]

CITIES = [
    "São Paulo", "Rio de Janeiro", "Brasília", "Salvador", "Fortaleza", "Belo Horizonte",
    "Manaus", "Curitiba", "Recife", "Porto Alegre", "Belém", "Goiânia", "Guarulhos",
    "Campinas", "São Luís", "Maceió", "Duque de Caxias", "Campo Grande", "Natal",
    "Teresina", "São Bernardo do Campo", "João Pessoa", "Osasco", "Jaboatão", "Uberlândia",
    "Contagem", "Sorocaba", "Aracaju", "Feira de Santana", "Cuiabá",
]

STREET_FORMATS = [
    "Rua {name}, {number}", "Avenida {name}, {number}",
    "Travessa {name}, {number}", "Alameda {name}, {number}",
]

POSTAL_CODE_FORMAT = "#####-###"
PHONE_FORMAT = "+55 (##) #####-####"
CURRENCY = "BRL"
CURRENCY_SYMBOL = "R$"

BANKS = [
    "Banco do Brasil", "Itaú Unibanco", "Bradesco", "Caixa Econômica Federal",
    "Santander Brasil", "Nubank", "Inter", "BTG Pactual", "Safra", "Sicoob",
    "Banco Original", "C6 Bank",
]

EMAIL_DOMAINS = ["gmail.com", "hotmail.com", "uol.com.br", "outlook.com", "bol.com.br", "terra.com.br"]
COMPANY_SUFFIXES = ["Ltda.", "S.A.", "ME", "EIRELI", "S/S", "EPP"]
COUNTRY_NAME = "Brasil"
NAME_ORDER = "given_family"



"""India locale data — hi_IN"""

FIRST_NAMES_MALE = [
    "Aarav", "Vivaan", "Aditya", "Vihaan", "Arjun", "Reyansh", "Sai", "Arnav", "Dhruv",
    "Kabir", "Rohan", "Harsh", "Ishaan", "Karthik", "Pranav", "Rahul", "Amit", "Vijay",
    "Suresh", "Rajesh", "Manoj", "Anil", "Sanjay", "Ravi", "Ajay", "Mukesh", "Deepak",
    "Vikram", "Nikhil", "Gaurav", "Sachin", "Tushar", "Varun", "Ashish", "Akash",
    "Kunal", "Rohit", "Siddharth", "Anand", "Mohit", "Naveen", "Ramesh", "Sunil", "Vinod",
    "Prakash", "Dinesh", "Ganesh", "Mahesh", "Naresh", "Rakesh", "Umesh", "Yogesh",
    "Abhishek", "Arun", "Bharat", "Chetan", "Devendra", "Girish", "Hitesh", "Jagdish",
    "Kamlesh", "Lalit", "Manish", "Nilesh", "Om", "Pankaj", "Rajendra", "Satish",
    "Tarun", "Uttam", "Vikas", "Yash", "Aman", "Bhavesh", "Dev", "Gopal", "Hemant",
    "Jatin", "Kiran", "Lokesh", "Mohan", "Nandan", "Pradeep", "Ravindra", "Shashank",
    "Krishna", "Shiva", "Aryan", "Rudra", "Advait", "Ishan", "Lakshya", "Parth", "Tanmay",
    "Vedant", "Yuvraj", "Anirudh", "Darshan", "Harish", "Kishore", "Madhav", "Raghav",
]

FIRST_NAMES_FEMALE = [
    "Saanvi", "Aanya", "Aadhya", "Aaradhya", "Ananya", "Pari", "Myra", "Sara", "Diya",
    "Prisha", "Neha", "Pooja", "Shreya", "Kavya", "Riya", "Priya", "Anjali", "Deepika",
    "Meera", "Nisha", "Sunita", "Anita", "Rekha", "Suman", "Geeta", "Lata", "Savita",
    "Usha", "Kamala", "Padma", "Radha", "Sita", "Lakshmi", "Saraswati", "Durga",
    "Kiran", "Rani", "Jaya", "Indira", "Shanti", "Nandini", "Pallavi", "Swati", "Divya",
    "Preeti", "Archana", "Rashmi", "Smita", "Varsha", "Asha", "Bhavna", "Chitra",
    "Gauri", "Heena", "Jyoti", "Komal", "Manisha", "Nirmala", "Parvati", "Rukmini",
    "Seema", "Tara", "Uma", "Vandana", "Yamini", "Zara", "Aditi", "Bhoomika", "Chaitra",
    "Devika", "Esha", "Fatima", "Gayatri", "Harini", "Isha", "Janaki", "Khushi",
    "Lavanya", "Maitri", "Navya", "Oviya", "Pranita", "Radhika", "Sahana", "Tanvi",
    "Urvi", "Vaishali", "Wriddhika", "Yakshini", "Aishwarya", "Bhavya", "Chandni",
    "Daksha", "Ekta", "Gargi", "Hemani", "Ira", "Juhi", "Kritika", "Madhuri", "Nandita",
]

LAST_NAMES = [
    "Sharma", "Verma", "Gupta", "Singh", "Kumar", "Patel", "Reddy", "Nair", "Iyer", "Joshi",
    "Mehta", "Shah", "Chopra", "Kapoor", "Malhotra", "Rao", "Das", "Pillai", "Bhat",
    "Mishra", "Pandey", "Tiwari", "Dubey", "Shukla", "Srivastava", "Agarwal", "Bansal",
    "Chauhan", "Dhawan", "Garg", "Jain", "Khanna", "Mahajan", "Naidu", "Patil",
    "Rathore", "Saxena", "Thakur", "Yadav", "Bose", "Chandra", "Dutta", "Ghosh",
    "Hegde", "Iyengar", "Kaur", "Kulkarni", "Menon", "Mukherjee", "Nambiar",
    "Parekh", "Raghavan", "Sengupta", "Trivedi", "Upadhyay", "Varma", "Wagh",
    "Agnihotri", "Bajpai", "Chaturvedi", "Deshmukh", "Gokhale", "Hazra", "Jha",
    "Krishnan", "Lal", "Mistry", "Narayan", "Oberoi", "Prasad", "Ranganathan",
    "Shetty", "Tandon", "Venkatesh", "Bhargava", "Dikshit", "Grover", "Johar",
    "Kohli", "Luthra", "Madan", "Nanda", "Purohit", "Rajan", "Sodhi", "Trehan",
    "Ahuja", "Bakshi", "Chawla", "Deshpande", "Gill", "Hegde", "Jena", "Khatri",
    "Mankar", "Naik", "Pandit", "Rana", "Sarkar", "Sood", "Talwar", "Vohra",
]

CITIES = [
    "Mumbai", "Delhi", "Bengaluru", "Hyderabad", "Chennai", "Kolkata", "Pune", "Ahmedabad",
    "Jaipur", "Lucknow", "Kochi", "Chandigarh", "Indore", "Coimbatore", "Surat",
    "Nagpur", "Visakhapatnam", "Bhopal", "Patna", "Vadodara", "Ghaziabad", "Ludhiana",
    "Agra", "Nashik", "Ranchi", "Faridabad", "Meerut", "Rajkot", "Varanasi", "Thiruvananthapuram",
]

STREET_FORMATS = [
    "{number}, {name} Road", "{number}, {name} Marg",
    "{number}, {name} Nagar", "Plot {number}, {name} Colony",
    "{number}/{number}, {name} Street", "{number}, {name} Layout",
]

POSTAL_CODE_FORMAT = "######"
PHONE_FORMAT = "+91 #####-#####"
CURRENCY = "INR"
CURRENCY_SYMBOL = "₹"

BANKS = [
    "State Bank of India", "HDFC Bank", "ICICI Bank", "Punjab National Bank",
    "Bank of Baroda", "Axis Bank", "Kotak Mahindra Bank", "Yes Bank",
    "IndusInd Bank", "Union Bank of India", "Canara Bank", "Bank of India",
]

EMAIL_DOMAINS = ["gmail.com", "yahoo.co.in", "rediffmail.com", "outlook.com", "hotmail.com", "protonmail.com"]
COMPANY_SUFFIXES = ["Pvt. Ltd.", "Ltd.", "LLP", "Industries", "Enterprises", "Solutions", "Technologies"]
COUNTRY_NAME = "India"
NAME_ORDER = "given_family"



"""United Kingdom locale data — en_GB"""

FIRST_NAMES_MALE = [
    "Oliver", "George", "Arthur", "Noah", "Muhammad", "Leo", "Oscar", "Harry", "Archie",
    "Jack", "Henry", "Charlie", "Freddie", "Theo", "Thomas", "James", "William", "Edward",
    "Alexander", "Daniel", "Alfie", "Jacob", "Isaac", "Ethan", "Joseph", "Joshua",
    "Sebastian", "Lucas", "Max", "Samuel", "Logan", "Benjamin", "Harrison", "Theodore",
    "Dylan", "Finley", "Adam", "Mason", "Elijah", "Toby", "Reuben", "Jayden", "Liam",
    "Harvey", "Hugo", "Tommy", "Louie", "Jude", "Mohammad", "Ronnie", "Bobby", "Frankie",
    "Stanley", "Teddy", "Albert", "Reggie", "Albie", "Elliot", "Zachary", "Luca",
    "Gabriel", "Connor", "Caleb", "Nathan", "Aaron", "Louis", "Kai", "Blake", "Luke",
    "Riley", "Robert", "Matthew", "Ryan", "Tyler", "Ben", "Callum", "Aiden", "Cameron",
    "Lewis", "Owen", "Declan", "Kyle", "Jamie", "Kieran", "Ross", "Craig", "Scott",
    "Gareth", "Stuart", "Darren", "Lee", "Mark", "Ian", "Paul", "Stephen", "Andrew",
    "Simon", "David", "Michael", "Christopher", "Jonathan", "Peter", "Richard",
]

FIRST_NAMES_FEMALE = [
    "Olivia", "Amelia", "Isla", "Ava", "Mia", "Ivy", "Lily", "Isabella", "Rosie", "Sophia",
    "Grace", "Florence", "Freya", "Emily", "Poppy", "Charlotte", "Eleanor", "Ella",
    "Evelyn", "Daisy", "Sienna", "Matilda", "Willow", "Phoebe", "Ruby", "Sophie",
    "Alice", "Isabelle", "Harper", "Maisie", "Luna", "Aria", "Emilia", "Penelope",
    "Scarlett", "Millie", "Violet", "Layla", "Elsie", "Chloe", "Jessica", "Maya",
    "Bonnie", "Imogen", "Lottie", "Hallie", "Bella", "Zara", "Hannah", "Thea",
    "Esme", "Abigail", "Orla", "Eva", "Georgia", "Eliza", "Holly", "Margot", "Molly",
    "Darcie", "Annabelle", "Erin", "Iris", "Lola", "Maeve", "Robyn", "Heidi", "Eloise",
    "Mabel", "Ayla", "Elizabeth", "Martha", "Jasmine", "Clara", "Arabella", "Darcey",
    "Bethany", "Katherine", "Lucy", "Katie", "Emma", "Leah", "Lauren", "Amy", "Rebecca",
    "Rachel", "Laura", "Sarah", "Jennifer", "Victoria", "Louise", "Gemma", "Kelly",
    "Nicola", "Karen", "Sharon", "Lisa", "Tracy", "Joanne", "Claire", "Helen",
]

LAST_NAMES = [
    "Smith", "Jones", "Williams", "Taylor", "Brown", "Davies", "Wilson", "Evans", "Thomas",
    "Roberts", "Johnson", "Walker", "Wright", "Robinson", "Thompson", "White", "Hughes",
    "Edwards", "Green", "Hall", "Lewis", "Harris", "Clarke", "Patel", "Jackson",
    "Wood", "Turner", "Martin", "Cooper", "Hill", "Ward", "Morris", "Moore", "Clark",
    "Lee", "King", "Baker", "Harrison", "Morgan", "Allen", "James", "Scott", "Phillips",
    "Watson", "Davis", "Parker", "Price", "Bennett", "Young", "Griffiths", "Mitchell",
    "Kelly", "Cook", "Carter", "Richardson", "Bailey", "Collins", "Bell", "Shaw", "Murphy",
    "Miller", "Cox", "Richards", "Khan", "Marshall", "Anderson", "Simpson", "Ellis",
    "Adams", "Singh", "Begum", "Wilkinson", "Foster", "Chapman", "Powell", "Webb",
    "Rogers", "Gray", "Mason", "Ali", "Hunt", "Hussain", "Campbell", "Matthews",
    "Owen", "Palmer", "Holmes", "Mills", "Barnes", "Knight", "Lloyd", "Butler",
    "Russell", "Barker", "Fisher", "Stevens", "Jenkins", "Murray", "Dixon", "Harvey",
]

CITIES = [
    "London", "Manchester", "Birmingham", "Leeds", "Glasgow", "Liverpool", "Bristol",
    "Edinburgh", "Sheffield", "Cardiff", "Nottingham", "Newcastle", "Brighton", "Oxford",
    "Cambridge", "Leicester", "Coventry", "Belfast", "Sunderland", "Wolverhampton",
    "Plymouth", "Southampton", "Reading", "Derby", "Aberdeen", "Swansea", "Exeter",
    "York", "Bath", "Canterbury",
]

STREET_FORMATS = [
    "{number} {name} Street", "{number} {name} Road", "{number} {name} Lane",
    "{number} {name} Close", "{number} {name} Crescent", "{number} {name} Gardens",
    "{number} {name} Terrace", "{number} {name} Mews",
]

POSTAL_CODE_FORMAT = "??# #??"
PHONE_FORMAT = "+44 #### ######"
CURRENCY = "GBP"
CURRENCY_SYMBOL = "£"

BANKS = [
    "Barclays", "HSBC", "Lloyds Banking Group", "NatWest", "Santander UK",
    "Nationwide", "TSB", "Metro Bank", "Monzo", "Starling Bank",
    "Halifax", "Royal Bank of Scotland",
]

EMAIL_DOMAINS = ["gmail.com", "outlook.com", "yahoo.co.uk", "btinternet.com", "sky.com", "hotmail.co.uk"]
COMPANY_SUFFIXES = ["Ltd", "PLC", "LLP", "Group", "& Co.", "Partners", "Holdings"]
COUNTRY_NAME = "United Kingdom"
NAME_ORDER = "given_family"



"""
Locale-specific reference data for employment, national IDs,
payment methods, phone carriers, and more.
"""

from __future__ import annotations

# --- Job titles by industry ---
JOB_TITLES: dict[str, list[str]] = {
    "tech": [
        "Software Engineer", "Senior Developer", "Tech Lead", "CTO", "DevOps Engineer",
        "Data Scientist", "Product Manager", "UX Designer", "QA Engineer", "System Architect",
        "Frontend Developer", "Backend Developer", "ML Engineer", "Security Analyst",
        "Cloud Engineer", "Mobile Developer", "Engineering Manager", "Scrum Master",
        "Database Administrator", "Site Reliability Engineer",
    ],
    "finance": [
        "Financial Analyst", "Investment Banker", "Portfolio Manager", "Risk Analyst",
        "Compliance Officer", "Auditor", "CFO", "Accountant", "Tax Advisor", "Trader",
        "Credit Analyst", "Wealth Manager", "Fund Manager", "Actuary", "Treasury Analyst",
        "Loan Officer", "Controller", "Financial Planner", "Underwriter", "Claims Adjuster",
    ],
    "health": [
        "Doctor", "Nurse", "Pharmacist", "Lab Technician", "Surgeon", "Dentist",
        "Radiologist", "Physical Therapist", "Anesthesiologist", "Paramedic",
        "Clinical Research Associate", "Hospital Administrator", "Dietician",
        "Optometrist", "Psychiatrist", "Veterinarian", "Biomedical Engineer",
        "Public Health Analyst", "Medical Coder", "Nursing Assistant",
    ],
    "retail": [
        "Store Manager", "Sales Associate", "Visual Merchandiser", "Buyer",
        "Inventory Manager", "District Manager", "Cashier", "Loss Prevention Officer",
        "E-commerce Manager", "Supply Chain Analyst", "Brand Manager",
        "Customer Service Representative", "Warehouse Manager", "Logistics Coordinator",
        "Retail Analyst", "Category Manager", "Area Sales Manager", "Floor Supervisor",
        "Personal Shopper", "Procurement Specialist",
    ],
    "food": [
        "Head Chef", "Sous Chef", "Line Cook", "Restaurant Manager", "Pastry Chef",
        "Food Safety Inspector", "Sommelier", "Barista", "Nutritionist",
        "Food Scientist", "Kitchen Manager", "Catering Manager", "Bartender",
        "Food Photographer", "Menu Developer", "Quality Control Manager",
        "Supply Chain Manager", "Brand Ambassador", "Franchise Manager", "Baker",
    ],
    "manufacturing": [
        "Production Manager", "Quality Engineer", "Plant Manager", "Process Engineer",
        "Mechanical Engineer", "Safety Manager", "Operations Director", "Maintenance Technician",
        "Supply Chain Manager", "Industrial Designer", "Tooling Engineer",
        "Production Planner", "Assembly Line Supervisor", "Welder", "CNC Operator",
        "Automation Engineer", "Lean Specialist", "Materials Manager",
        "Environmental Health Officer", "Electrical Engineer",
    ],
}

# General / cross-industry titles
JOB_TITLES["general"] = [
    "CEO", "COO", "VP Operations", "HR Manager", "Marketing Director",
    "Office Manager", "Executive Assistant", "Receptionist", "Legal Counsel",
    "Business Analyst", "Project Manager", "Communications Director",
    "Administrative Assistant", "Consultant", "Intern", "Freelancer",
    "Sales Director", "Training Manager", "IT Support", "Janitor",
]

# --- Department names ---
DEPARTMENTS = [
    "Engineering", "Finance", "Human Resources", "Marketing", "Sales",
    "Operations", "Legal", "Customer Support", "Research & Development",
    "Quality Assurance", "Product", "Design", "IT", "Procurement",
    "Administration", "Strategy", "Business Development", "Compliance",
]

# --- Salary ranges by locale (annual, in local currency) ---
SALARY_RANGES: dict[str, tuple[float, float, float]] = {
    # (low, median, high)
    "US": (35000, 65000, 200000),
    "JP": (3000000, 5500000, 15000000),
    "DE": (30000, 52000, 140000),
    "BR": (24000, 60000, 240000),
    "IN": (300000, 800000, 3500000),
    "GB": (25000, 42000, 130000),
    "FR": (28000, 45000, 120000),
    "KR": (30000000, 50000000, 120000000),
    "ES": (20000, 33000, 90000),
    "SA": (60000, 144000, 480000),
    "NG": (1200000, 4800000, 18000000),
}

# --- National ID formats ---
# '#' = digit, '?' = letter, '*' = alphanumeric
NATIONAL_ID_FORMATS: dict[str, dict[str, str]] = {
    "US": {"name": "SSN", "format": "###-##-####"},
    "JP": {"name": "My Number", "format": "#### #### ####"},
    "DE": {"name": "Personalausweis", "format": "?#########"},
    "BR": {"name": "CPF", "format": "###.###.###-##"},
    "IN": {"name": "Aadhaar", "format": "#### #### ####"},
    "GB": {"name": "NI Number", "format": "?? ## ## ## ?"},
    "FR": {"name": "INSEE", "format": "# ## ## ?? ### ###"},
    "KR": {"name": "RRN", "format": "######-#######"},
    "ES": {"name": "DNI", "format": "########?"},
    "SA": {"name": "National ID", "format": "##########"},
    "NG": {"name": "NIN", "format": "###########"},
}

# --- Payment methods by locale ---
PAYMENT_METHODS: dict[str, list[tuple[str, float]]] = {
    # (method_name, weight)
    "US": [("Credit Card", 40), ("Debit Card", 25), ("Zelle", 12), ("Venmo", 8), ("PayPal", 10), ("Cash", 5)],
    "JP": [("Credit Card", 30), ("Cash", 25), ("PayPay", 15), ("Suica/IC Card", 12), ("Debit Card", 8), ("Bank Transfer", 10)],
    "DE": [("Bank Transfer", 30), ("Cash", 20), ("EC Card", 20), ("PayPal", 15), ("Credit Card", 10), ("Klarna", 5)],
    "BR": [("Pix", 40), ("Credit Card", 25), ("Debit Card", 15), ("Boleto", 12), ("Cash", 8)],
    "IN": [("UPI", 45), ("Debit Card", 15), ("Credit Card", 12), ("Net Banking", 10), ("Cash", 10), ("Wallet", 8)],
    "GB": [("Debit Card", 35), ("Credit Card", 25), ("Bank Transfer", 15), ("Apple Pay", 10), ("PayPal", 10), ("Cash", 5)],
    "FR": [("Credit Card", 35), ("Bank Transfer", 20), ("Cash", 15), ("PayPal", 12), ("Cheque", 10), ("Apple Pay", 8)],
    "KR": [("Credit Card", 40), ("Samsung Pay", 15), ("Kakao Pay", 15), ("Bank Transfer", 12), ("Naver Pay", 10), ("Cash", 8)],
    "ES": [("Credit Card", 30), ("Debit Card", 25), ("Bank Transfer", 20), ("Bizum", 12), ("Cash", 8), ("PayPal", 5)],
    "SA": [("Mada Card", 35), ("Credit Card", 25), ("STC Pay", 15), ("Bank Transfer", 15), ("Cash", 10)],
    "NG": [("Bank Transfer", 30), ("POS/Debit Card", 20), ("Cash", 20), ("Opay", 12), ("Mobile Money", 10), ("USSD", 8)],
}

# --- Phone carriers by locale ---
PHONE_CARRIERS: dict[str, list[str]] = {
    "US": ["Verizon", "AT&T", "T-Mobile", "Mint Mobile", "Cricket", "Visible"],
    "JP": ["NTT Docomo", "au (KDDI)", "SoftBank", "Rakuten Mobile", "Y!mobile", "UQ Mobile"],
    "DE": ["Telekom", "Vodafone", "O2", "1&1", "Congstar", "ALDI TALK"],
    "BR": ["Vivo", "Claro", "TIM", "Oi", "Algar Telecom"],
    "IN": ["Jio", "Airtel", "Vi (Vodafone Idea)", "BSNL", "MTNL"],
    "GB": ["EE", "Vodafone UK", "Three", "O2 UK", "Sky Mobile", "Tesco Mobile"],
    "FR": ["Orange", "SFR", "Bouygues Telecom", "Free Mobile", "La Poste Mobile"],
    "KR": ["SK Telecom", "KT", "LG U+"],
    "ES": ["Movistar", "Vodafone", "Orange", "MásMóvil", "Yoigo"],
    "SA": ["STC", "Mobily", "Zain", "Virgin Mobile SA"],
    "NG": ["MTN Nigeria", "Airtel Nigeria", "Glo", "9mobile"],
}

# --- Tax ID formats (company) ---
TAX_ID_FORMATS: dict[str, dict[str, str]] = {
    "US": {"name": "EIN", "format": "##-#######"},
    "JP": {"name": "Corporate Number", "format": "#############"},
    "DE": {"name": "USt-IdNr", "format": "DE#########"},
    "BR": {"name": "CNPJ", "format": "##.###.###/####-##"},
    "IN": {"name": "GSTIN", "format": "##?????####?#??"},
    "GB": {"name": "VAT Number", "format": "GB### #### ##"},
    "FR": {"name": "SIRET", "format": "### ### ### #####"},
    "KR": {"name": "BRN", "format": "###-##-#####"},
    "ES": {"name": "NIF", "format": "?########"},
    "SA": {"name": "Tax Number", "format": "###############"},
    "NG": {"name": "TIN", "format": "##########-####"},
}

# --- Credit card preferences by locale ---
CREDIT_CARD_TYPES: dict[str, list[tuple[str, float]]] = {
    "US": [("Visa", 40), ("Mastercard", 30), ("Amex", 15), ("Discover", 10), ("Other", 5)],
    "JP": [("JCB", 35), ("Visa", 30), ("Mastercard", 20), ("Amex", 10), ("Diners", 5)],
    "DE": [("Visa", 35), ("Mastercard", 35), ("Girocard", 20), ("Amex", 10)],
    "BR": [("Visa", 35), ("Mastercard", 30), ("Elo", 25), ("Amex", 5), ("Hipercard", 5)],
    "IN": [("Visa", 30), ("Mastercard", 25), ("RuPay", 35), ("Amex", 5), ("Diners", 5)],
    "GB": [("Visa", 45), ("Mastercard", 40), ("Amex", 10), ("Other", 5)],
    "FR": [("Visa", 45), ("Mastercard", 35), ("Carte Bancaire", 10), ("Amex", 10)],
    "KR": [("Samsung Card", 20), ("Shinhan Card", 20), ("KB Card", 20), ("Visa", 15), ("Mastercard", 15), ("Amex", 10)],
    "ES": [("Visa", 45), ("Mastercard", 40), ("Amex", 10), ("Other", 5)],
    "SA": [("Visa", 35), ("Mastercard", 35), ("Mada", 25), ("Amex", 5)],
    "NG": [("Visa", 40), ("Mastercard", 35), ("Verve", 20), ("Amex", 5)],
}

# --- Height/Weight ranges by locale (cm, kg) ---
BIOMETRICS: dict[str, dict[str, tuple[float, float, float]]] = {
    # (low, mean, high) for each
    "US": {"height_m": (165, 177, 193), "height_f": (152, 163, 175), "weight_m": (65, 90, 120), "weight_f": (50, 75, 105)},
    "JP": {"height_m": (160, 172, 184), "height_f": (148, 158, 170), "weight_m": (55, 70, 90), "weight_f": (42, 55, 72)},
    "DE": {"height_m": (170, 180, 195), "height_f": (158, 167, 180), "weight_m": (68, 85, 110), "weight_f": (52, 70, 95)},
    "BR": {"height_m": (163, 174, 188), "height_f": (152, 161, 174), "weight_m": (60, 78, 100), "weight_f": (48, 65, 88)},
    "IN": {"height_m": (158, 170, 183), "height_f": (148, 157, 168), "weight_m": (52, 70, 92), "weight_f": (42, 58, 78)},
    "GB": {"height_m": (168, 178, 192), "height_f": (155, 164, 176), "weight_m": (65, 84, 110), "weight_f": (50, 72, 98)},
    "FR": {"height_m": (168, 178, 190), "height_f": (155, 165, 176), "weight_m": (63, 80, 105), "weight_f": (48, 65, 88)},
    "KR": {"height_m": (165, 175, 187), "height_f": (153, 162, 173), "weight_m": (58, 73, 95), "weight_f": (44, 57, 75)},
    "ES": {"height_m": (165, 176, 190), "height_f": (155, 163, 176), "weight_m": (62, 80, 105), "weight_f": (48, 65, 88)},
    "SA": {"height_m": (163, 174, 188), "height_f": (150, 160, 172), "weight_m": (60, 82, 110), "weight_f": (48, 68, 95)},
    "NG": {"height_m": (162, 172, 186), "height_f": (150, 160, 172), "weight_m": (55, 73, 98), "weight_f": (45, 65, 90)},
}

# --- University names by locale ---
UNIVERSITIES: dict[str, list[str]] = {
    "US": [
        "MIT", "Stanford", "Harvard", "UC Berkeley", "Georgia Tech", "UT Austin",
        "University of Michigan", "UCLA", "Columbia", "NYU", "Ohio State",
        "Penn State", "University of Florida", "University of Washington",
        "Carnegie Mellon", "Duke", "Caltech", "Northwestern", "USC", "Purdue",
    ],
    "JP": [
        "東京大学", "京都大学", "大阪大学", "東北大学", "名古屋大学", "九州大学",
        "北海道大学", "早稲田大学", "慶應義塾大学", "東京工業大学", "筑波大学",
        "一橋大学", "上智大学", "明治大学", "同志社大学", "立命館大学",
    ],
    "DE": [
        "TU München", "LMU München", "Heidelberg", "Humboldt Berlin", "TU Berlin",
        "RWTH Aachen", "Universität Freiburg", "Universität Hamburg", "TU Dresden",
        "Universität Bonn", "Universität Göttingen", "Universität Köln",
    ],
    "BR": [
        "USP", "UNICAMP", "UFRJ", "UFMG", "UFRGS", "UnB", "UFPE", "UFSC",
        "PUC-Rio", "FGV", "Insper", "UFPR", "UFC", "UNESP", "UFBA",
    ],
    "IN": [
        "IIT Bombay", "IIT Delhi", "IIT Madras", "IIT Kanpur", "IISc Bangalore",
        "BITS Pilani", "IIT Kharagpur", "IIT Roorkee", "NIT Trichy", "Delhi University",
        "JNU", "Anna University", "Jadavpur University", "VIT", "Manipal",
        "Amity University", "Symbiosis", "Christ University", "SRM", "KIIT",
    ],
    "GB": [
        "Oxford", "Cambridge", "Imperial College", "UCL", "Edinburgh", "Manchester",
        "King's College London", "LSE", "Bristol", "Warwick", "Glasgow", "Leeds",
        "Durham", "Birmingham", "Nottingham", "Sheffield", "Southampton", "Exeter",
    ],
    "FR": [
        "Sorbonne", "École Polytechnique", "ENS Paris", "HEC Paris", "Sciences Po",
        "INSEAD", "Université Paris-Saclay", "Université de Lyon", "Université de Strasbourg",
        "Université de Bordeaux", "Centrale Paris", "ESSEC", "EDHEC",
    ],
    "KR": [
        "서울대학교", "KAIST", "연세대학교", "고려대학교", "포항공과대학교",
        "성균관대학교", "한양대학교", "서강대학교", "중앙대학교", "경희대학교",
        "이화여자대학교", "건국대학교", "동국대학교", "홍익대학교",
    ],
    "ES": [
        "Universidad Complutense", "Universidad de Barcelona", "Universidad Autónoma de Madrid",
        "Universidad de Salamanca", "Universidad de Valencia", "Universidad de Sevilla",
        "Universidad de Granada", "IE University", "ESADE", "Universidad Carlos III",
    ],
    "SA": [
        "King Saud University", "KAUST", "King Abdulaziz University",
        "King Fahd University", "Imam Muhammad University", "Princess Nourah University",
        "Umm Al-Qura University", "Taibah University", "King Khalid University",
    ],
    "NG": [
        "University of Lagos", "University of Ibadan", "Obafemi Awolowo University",
        "University of Nigeria Nsukka", "Ahmadu Bello University", "Covenant University",
        "University of Benin", "Lagos State University", "University of Ilorin",
        "Babcock University", "University of Port Harcourt", "FUTA",
    ],
}

DEGREE_TYPES = [
    ("Bachelor's", 55), ("Master's", 25), ("PhD", 5), ("Associate's", 10), ("Diploma", 5),
]

DEGREE_FIELDS = [
    "Computer Science", "Business Administration", "Engineering", "Economics",
    "Medicine", "Law", "Psychology", "Mathematics", "Physics", "Chemistry",
    "Biology", "English Literature", "History", "Political Science", "Accounting",
    "Marketing", "Finance", "Architecture", "Design", "Education",
    "Electrical Engineering", "Mechanical Engineering", "Civil Engineering",
    "Pharmacy", "Nursing", "Communication", "Sociology", "Philosophy",
]

# --- Vehicle data by locale ---
CAR_BRANDS: dict[str, list[tuple[str, float]]] = {
    "US": [("Ford", 18), ("Chevrolet", 15), ("Toyota", 14), ("Honda", 12), ("Tesla", 8), ("Jeep", 7), ("Nissan", 6), ("Hyundai", 5), ("BMW", 5), ("Ram", 5), ("Subaru", 5)],
    "JP": [("Toyota", 30), ("Honda", 18), ("Nissan", 12), ("Suzuki", 10), ("Mazda", 8), ("Subaru", 6), ("Daihatsu", 6), ("Mitsubishi", 5), ("Lexus", 5)],
    "DE": [("Volkswagen", 22), ("BMW", 12), ("Mercedes-Benz", 12), ("Audi", 10), ("Opel", 8), ("Ford", 7), ("Skoda", 6), ("Hyundai", 5), ("Toyota", 5), ("Seat", 5), ("Porsche", 4), ("Mini", 4)],
    "BR": [("Fiat", 22), ("Volkswagen", 18), ("Chevrolet", 16), ("Hyundai", 12), ("Toyota", 10), ("Renault", 8), ("Honda", 6), ("Jeep", 5), ("Nissan", 3)],
    "IN": [("Maruti Suzuki", 35), ("Hyundai", 18), ("Tata", 15), ("Mahindra", 10), ("Kia", 8), ("Toyota", 5), ("Honda", 4), ("MG", 3), ("Renault", 2)],
    "GB": [("Ford", 15), ("Volkswagen", 10), ("Vauxhall", 10), ("BMW", 8), ("Audi", 8), ("Mercedes", 7), ("Toyota", 7), ("Nissan", 6), ("Kia", 6), ("Hyundai", 6), ("Range Rover", 5), ("Mini", 5), ("Tesla", 4), ("Volvo", 3)],
    "FR": [("Peugeot", 20), ("Renault", 18), ("Citroën", 12), ("Dacia", 8), ("Volkswagen", 7), ("Toyota", 7), ("BMW", 5), ("Mercedes", 5), ("Audi", 5), ("Ford", 5), ("Fiat", 4), ("Opel", 4)],
    "KR": [("Hyundai", 35), ("Kia", 30), ("Genesis", 10), ("Samsung (Renault)", 8), ("BMW", 5), ("Mercedes", 5), ("Tesla", 4), ("Audi", 3)],
    "ES": [("Seat", 15), ("Volkswagen", 12), ("Peugeot", 10), ("Renault", 10), ("Toyota", 8), ("Citroën", 7), ("Ford", 7), ("Hyundai", 6), ("Kia", 6), ("BMW", 5), ("Dacia", 5), ("Audi", 5), ("Mercedes", 4)],
    "SA": [("Toyota", 30), ("Hyundai", 15), ("Nissan", 10), ("Ford", 8), ("Chevrolet", 8), ("Kia", 7), ("Honda", 6), ("Lexus", 5), ("BMW", 5), ("Mercedes", 6)],
    "NG": [("Toyota", 35), ("Honda", 15), ("Mercedes-Benz", 10), ("Lexus", 10), ("Hyundai", 8), ("Kia", 6), ("Ford", 5), ("Nissan", 5), ("Innoson", 3), ("Peugeot", 3)],
}

# --- Insurance providers by locale ---
INSURANCE_PROVIDERS: dict[str, list[str]] = {
    "US": ["Blue Cross", "UnitedHealth", "Aetna", "Cigna", "Humana", "Kaiser Permanente"],
    "JP": ["National Health Insurance", "Social Insurance", "Zenkyoren", "Nippon Life"],
    "DE": ["TK", "AOK", "Barmer", "DAK", "IKK", "Allianz Private"],
    "BR": ["SUS", "Amil", "Bradesco Saúde", "SulAmérica", "Unimed", "Hapvida"],
    "IN": ["LIC", "Star Health", "HDFC Ergo", "ICICI Lombard", "Max Bupa", "Ayushman Bharat"],
    "GB": ["NHS", "Bupa", "AXA Health", "Vitality", "Aviva", "Nuffield Health"],
    "FR": ["Sécurité Sociale", "Harmonie Mutuelle", "MGEN", "AXA", "Groupama", "Malakoff Humanis"],
    "KR": ["National Health Insurance", "Samsung Life", "Hanwha Life", "Kyobo Life"],
    "ES": ["Seguridad Social", "Sanitas", "Adeslas", "Mapfre", "DKV", "Asisa"],
    "SA": ["Bupa Arabia", "Tawuniya", "Medgulf", "Al Rajhi Takaful", "AXA Cooperative"],
    "NG": ["NHIS", "Leadway Health", "Hygeia", "AXA Mansard", "Reliance HMO"],
}



"""
World — the high-level API for generating entire coherent datasets.

A World is a self-contained universe of mock data. Create a world,
add companies and people, and everything stays internally consistent.

    world = World(locale="ja_JP", seed=42)
    company = world.company(industry="tech")
    employees = company.hire(10)
    products = world.products(5, company=company)
    transactions = world.transactions(100)
"""

from __future__ import annotations

import json
from typing import Optional

from fabrikate.context import Context
from fabrikate.generators.company import Company
from fabrikate.generators.person import Person
from fabrikate.generators.product import Product
from fabrikate.generators.transaction import Transaction


class World:
    """
    A coherent mock data universe.

    Parameters
    ----------
    locale : str
        Locale code (e.g., "ja_JP", "en_US", "de_DE", "pt_BR").
    seed : int or None
        Random seed for full reproducibility.
    year : int
        Reference year for dates and ages.
    industry : str or None
        Default industry for companies/products.
    """

    def __init__(
        self,
        locale: str = "en_US",
        seed: Optional[int] = None,
        year: int = 2025,
        industry: Optional[str] = None,
    ):
        self.ctx = Context(
            locale=locale,
            seed=seed,
            year=year,
            industry=industry,
        )

    def person(self, **overrides) -> Person:
        """Generate a single coherent person."""
        return Person.generate(self.ctx, **overrides)

    def people(self, count: int, **overrides) -> list[Person]:
        """Generate multiple coherent people."""
        return [Person.generate(self.ctx, **overrides) for _ in range(count)]

    def company(self, **overrides) -> Company:
        """Generate a single coherent company."""
        return Company.generate(self.ctx, **overrides)

    def companies(self, count: int, **overrides) -> list[Company]:
        """Generate multiple coherent companies."""
        return [Company.generate(self.ctx, **overrides) for _ in range(count)]

    def product(
        self,
        company: Optional[Company] = None,
        **overrides,
    ) -> Product:
        """Generate a product, optionally tied to a company."""
        return Product.generate(
            self.ctx,
            company_id=company.id if company else None,
            industry=company.industry if company else None,
            **overrides,
        )

    def products(
        self,
        count: int,
        company: Optional[Company] = None,
        **overrides,
    ) -> list[Product]:
        """Generate multiple products."""
        return [self.product(company=company, **overrides) for _ in range(count)]

    def transaction(
        self,
        sender: Optional[Person] = None,
        receiver: Optional[Person | Company] = None,
        product: Optional[Product] = None,
        **overrides,
    ) -> Transaction:
        """Generate a transaction between entities."""
        return Transaction.generate(
            self.ctx,
            sender_id=sender.id if sender else None,
            receiver_id=receiver.id if receiver else None,
            product_id=product.id if product else None,
            **overrides,
        )

    def transactions(
        self,
        count: int,
        sender: Optional[Person] = None,
        receiver: Optional[Person | Company] = None,
        **overrides,
    ) -> list[Transaction]:
        """Generate multiple transactions."""
        return [
            self.transaction(sender=sender, receiver=receiver, **overrides)
            for _ in range(count)
        ]

    def dataset(self) -> dict:
        """
        Export all registered entities as a structured dictionary.

        Useful for dumping the entire world to JSON for test fixtures.
        """
        people = [e.to_dict() for e in self.ctx.find(Person)]
        companies = [e.to_dict() for e in self.ctx.find(Company)]
        products = [e.to_dict() for e in self.ctx.find(Product)]
        txns = [e.to_dict() for e in self.ctx.find(Transaction)]

        return {
            "metadata": {
                "locale": self.ctx.locale,
                "seed": self.ctx.seed,
                "year": self.ctx.year,
                "industry": self.ctx.industry,
            },
            "people": people,
            "companies": companies,
            "products": products,
            "transactions": txns,
        }

    def to_json(self, indent: int = 2) -> str:
        """Export the entire world as a JSON string."""
        return json.dumps(self.dataset(), indent=indent, ensure_ascii=False)



"""
Product generator — creates products tied to a company and industry.
"""

from __future__ import annotations

import uuid
from dataclasses import dataclass, field
from typing import Optional

from fabrikate.context import Context
from fabrikate.locales import get_locale


PRODUCT_TEMPLATES = {
    "tech": {
        "prefixes": ["Smart", "Cloud", "AI", "Quantum", "Edge", "Hyper"],
        "names": ["Platform", "Suite", "Engine", "Hub", "Console", "Workspace"],
        "price_range": (29.99, 999.99),
    },
    "finance": {
        "prefixes": ["Wealth", "Capital", "Trade", "Fund", "Secure", "Prime"],
        "names": ["Account", "Portfolio", "Plan", "Service", "Advisory", "Shield"],
        "price_range": (9.99, 499.99),
    },
    "health": {
        "prefixes": ["Vita", "Bio", "Pure", "Active", "Natural", "Care"],
        "names": ["Plus", "Formula", "Complex", "Boost", "System", "Daily"],
        "price_range": (12.99, 149.99),
    },
    "retail": {
        "prefixes": ["Home", "Fresh", "Select", "Choice", "Value", "Premium"],
        "names": ["Collection", "Line", "Series", "Bundle", "Set", "Pack"],
        "price_range": (4.99, 299.99),
    },
    "food": {
        "prefixes": ["Farm", "Artisan", "Golden", "Harvest", "Organic", "Wild"],
        "names": ["Blend", "Reserve", "Selection", "Batch", "Origin", "Craft"],
        "price_range": (3.99, 79.99),
    },
    "manufacturing": {
        "prefixes": ["Industrial", "Heavy", "Precision", "Titan", "Forge", "Pro"],
        "names": ["Series", "Grade", "Class", "Spec", "Line", "Mark"],
        "price_range": (99.99, 9999.99),
    },
}


@dataclass
class Product:
    """A product tied to a company and industry context."""

    id: str = ""
    name: str = ""
    sku: str = ""
    price: float = 0.0
    currency: str = ""
    category: str = ""
    company_id: Optional[str] = None
    _ctx: Optional[Context] = field(default=None, repr=False)

    @classmethod
    def generate(
        cls,
        ctx: Context,
        company_id: Optional[str] = None,
        industry: Optional[str] = None,
        **overrides,
    ) -> Product:
        locale = get_locale(ctx.locale)
        rng = ctx._rng

        product = cls(_ctx=ctx)
        product.id = overrides.get("id", str(uuid.UUID(int=rng.getrandbits(128))))

        ind = industry or ctx.industry or "tech"
        templates = PRODUCT_TEMPLATES.get(ind, PRODUCT_TEMPLATES["tech"])

        prefix = rng.choice(templates["prefixes"])
        name = rng.choice(templates["names"])
        product.name = overrides.get("name", f"{prefix} {name}")
        product.category = ind
        product.company_id = company_id

        # SKU — readable product code
        product.sku = overrides.get(
            "sku",
            f"{prefix[:3].upper()}-{rng.randint(1000, 9999)}",
        )

        # Price — industry-appropriate range, in locale currency
        low, high = templates["price_range"]
        product.price = overrides.get(
            "price", round(rng.uniform(low, high), 2)
        )
        product.currency = locale.currency

        ctx.register(product)
        return product

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "name": self.name,
            "sku": self.sku,
            "price": self.price,
            "currency": self.currency,
            "category": self.category,
            "company_id": self.company_id,
        }



"""Nigeria locale data — en_NG"""

FIRST_NAMES_MALE = [
    "Chinedu", "Oluwaseun", "Emeka", "Tunde", "Ifeanyi", "Obinna", "Adebayo", "Chibueze",
    "Ayomide", "Olumide", "Yusuf", "Ibrahim", "Chukwuma", "Damilare", "Ebuka", "Kelechi",
    "Tochukwu", "Abiodun", "Femi", "Uche", "Olamide", "Babajide", "Chinonso", "Adekunle",
    "Ikechukwu", "Nnamdi", "Olalekan", "Temitope", "Gbenga", "Segun", "Musa", "Abdullahi",
    "Aliyu", "Aminu", "Bashir", "Danjuma", "Garba", "Hamza", "Idris", "Jibril",
    "Kabiru", "Lawal", "Murtala", "Nuhu", "Suleiman", "Usman", "Yakubu", "Abubakar",
    "Chidi", "Chijioke", "Chuka", "Ekene", "Ifeanyichukwu", "Jideofor", "Kenechukwu",
    "Lotachukwu", "Munachi", "Nnaemeka", "Obichukwu", "Odinaka", "Onyeka", "Somtochukwu",
    "Tobechukwu", "Uchechukwu", "Chukwuemeka", "Adewale", "Akintunde", "Bamidele",
    "Boluwatife", "Dapo", "Folarin", "Kolawole", "Oluwadamilola", "Oluwatobi",
    "Oluwatosin", "Rotimi", "Saheed", "Taiwo", "Kehinde", "Wale", "Ayo", "Kunle",
    "Niyi", "Sola", "Dele", "Lanre", "Lekan", "Toyin", "Jide", "Tayo", "Bode",
    "Kayode", "Adeniyi", "Olayinka", "Olufemi", "Olusegun", "Olusola", "Oluwaferanmi",
    "Adedayo", "Goodluck", "Sunday",
]

FIRST_NAMES_FEMALE = [
    "Chioma", "Ngozi", "Amara", "Oluwabunmi", "Funke", "Adaeze", "Yetunde", "Ifeoma",
    "Blessing", "Aisha", "Chiamaka", "Nneka", "Halima", "Titilayo", "Zainab", "Kemi",
    "Nkechi", "Folake", "Ada", "Ogechi", "Onyinyechi", "Bukola", "Nkem", "Adaugo",
    "Ebere", "Uchenna", "Chinwe", "Adaora", "Nwanneka", "Oluchi", "Chinelo", "Somkene",
    "Amarachi", "Obiageli", "Njideka", "Oluwatoyin", "Oluwaseun", "Adenike", "Abimbola",
    "Jumoke", "Modupe", "Omolara", "Opeyemi", "Tolulope", "Temilolu", "Boluwatife",
    "Oluwadamilola", "Oluwadarasimi", "Morayo", "Morenike", "Yewande", "Eniola",
    "Ifeoluwa", "Oyinkansola", "Ayomide", "Moyinoluwa", "Iretiola", "Damilola",
    "Hadiza", "Hauwa", "Fatima", "Amina", "Maryam", "Nafisa", "Rahma", "Safiya",
    "Asmau", "Bilkisu", "Firdaus", "Hajara", "Khadija", "Salma", "Sumayya", "Ummi",
    "Grace", "Joy", "Peace", "Faith", "Hope", "Mercy", "Patience", "Glory",
    "Esther", "Deborah", "Hannah", "Ruth", "Naomi", "Abigail", "Eunice", "Dorcas",
    "Priscilla", "Lydia", "Comfort", "Precious", "Favour", "Gift", "Happiness",
    "Love", "Wisdom", "Divine", "Princess", "Queen",
]

LAST_NAMES = [
    "Okonkwo", "Adeyemi", "Ibrahim", "Okafor", "Balogun", "Nwosu", "Adebayo", "Eze",
    "Mohammed", "Abubakar", "Nnamdi", "Obi", "Yusuf", "Adeleke", "Chukwu", "Adeola",
    "Musa", "Okeke", "Lawal", "Onuoha", "Ogundimu", "Usman", "Ogundipe", "Olowu",
    "Akinyemi", "Chukwuemeka", "Ojo", "Afolabi", "Nwachukwu", "Aliyu", "Oladipo",
    "Ogunyemi", "Babangida", "Nwafor", "Oyelaran", "Oladele", "Adebiyi", "Uzoma",
    "Okonkwu", "Bello", "Danjuma", "Audu", "Garba", "Sani", "Abdullahi", "Gambo",
    "Shehu", "Tanko", "Waziri", "Dikko", "Ciroma", "Ringim", "Gwarzo", "Hadejia",
    "Aigbe", "Oghene", "Efosa", "Osaze", "Igbinovia", "Osakwe", "Osagie", "Idemudia",
    "Emovon", "Irabor", "Okoh", "Peterside", "Nwokolo", "Anyanwu", "Iheanacho",
    "Obiora", "Achebe", "Adichie", "Ekwueme", "Azikiwe", "Ojukwu", "Ironsi", "Dimka",
    "Babatunde", "Obasanjo", "Awolowo", "Soyinka", "Kuti", "Anikulapo", "Ransome",
    "Solarin", "Awojobi", "Adesanya", "Fashola", "Sanwo-Olu", "Ambode", "Tinubu",
    "Jakande", "Lateef", "Dosunmu", "Ojora", "Elegushi", "Oniru", "Akiolu", "Rilwan",
]

CITIES = [
    "Lagos", "Abuja", "Port Harcourt", "Kano", "Ibadan", "Benin City", "Enugu", "Kaduna",
    "Calabar", "Warri", "Owerri", "Jos", "Abeokuta", "Uyo", "Asaba", "Maiduguri",
    "Ilorin", "Onitsha", "Aba", "Sokoto", "Akure", "Osogbo", "Bauchi", "Lokoja",
    "Yola", "Lafia", "Makurdi", "Minna", "Abakaliki", "Ado-Ekiti",
]

STREET_FORMATS = [
    "{number} {name} Street", "{number} {name} Avenue",
    "{number} {name} Road", "{number} {name} Close",
    "{number} {name} Crescent", "{number} {name} Drive",
]

POSTAL_CODE_FORMAT = "######"
PHONE_FORMAT = "+234 ### ### ####"
CURRENCY = "NGN"
CURRENCY_SYMBOL = "₦"

BANKS = [
    "First Bank of Nigeria", "Zenith Bank", "GTBank", "Access Bank", "UBA",
    "Stanbic IBTC", "Fidelity Bank", "Sterling Bank", "Wema Bank", "Opay",
    "Kuda Bank", "Polaris Bank",
]

EMAIL_DOMAINS = ["gmail.com", "yahoo.com", "hotmail.com", "outlook.com", "protonmail.com", "aol.com"]
COMPANY_SUFFIXES = ["Ltd.", "PLC", "& Sons", "Enterprises", "Nigeria Ltd.", "Group", "& Co."]
COUNTRY_NAME = "Nigeria"
NAME_ORDER = "given_family"



"""United States locale data — en_US"""

FIRST_NAMES_MALE = [
    "James", "Robert", "Michael", "William", "David", "Richard", "Joseph", "Thomas",
    "Christopher", "Daniel", "Matthew", "Anthony", "Mark", "Steven", "Andrew",
    "Paul", "Joshua", "Kenneth", "Kevin", "Brian", "Timothy", "Ronald", "George",
    "Jason", "Edward", "Jeffrey", "Ryan", "Jacob", "Gary", "Nicholas",
    "Eric", "Jonathan", "Stephen", "Larry", "Justin", "Scott", "Brandon", "Benjamin",
    "Samuel", "Raymond", "Gregory", "Frank", "Alexander", "Patrick", "Jack", "Dennis",
    "Jerry", "Tyler", "Aaron", "Nathan", "Henry", "Peter", "Douglas", "Zachary",
    "Kyle", "Noah", "Harold", "Carl", "Gerald", "Keith", "Logan", "Eugene",
    "Russell", "Randy", "Philip", "Harry", "Vincent", "Bobby", "Dylan", "Johnny",
    "Jesse", "Roy", "Billy", "Bruce", "Gabriel", "Joe", "Jordan", "Ralph",
    "Eugene", "Randy", "Wayne", "Elijah", "Oscar", "Landon", "Caleb", "Mason",
    "Ethan", "Aiden", "Lucas", "Jackson", "Sebastian", "Levi", "Mateo", "Owen",
    "Carter", "Jayden", "Luke", "Hunter", "Wyatt", "Isaac",
]

FIRST_NAMES_FEMALE = [
    "Mary", "Patricia", "Jennifer", "Linda", "Barbara", "Elizabeth", "Susan", "Jessica",
    "Sarah", "Karen", "Lisa", "Nancy", "Betty", "Dorothy", "Sandra", "Ashley",
    "Margaret", "Emily", "Donna", "Michelle", "Carol", "Amanda", "Melissa", "Deborah",
    "Stephanie", "Rebecca", "Sharon", "Laura", "Cynthia", "Kathleen", "Amy", "Angela",
    "Shirley", "Brenda", "Anna", "Pamela", "Emma", "Nicole", "Helen", "Samantha",
    "Katherine", "Christine", "Debra", "Rachel", "Carolyn", "Janet", "Catherine",
    "Maria", "Heather", "Diane", "Ruth", "Julie", "Olivia", "Joyce", "Virginia",
    "Victoria", "Kelly", "Lauren", "Christina", "Joan", "Evelyn", "Judith", "Megan",
    "Andrea", "Cheryl", "Hannah", "Jacqueline", "Martha", "Gloria", "Teresa", "Ann",
    "Sara", "Madison", "Frances", "Kathryn", "Janice", "Jean", "Abigail", "Alice",
    "Julia", "Judy", "Sophia", "Grace", "Denise", "Amber", "Doris", "Marilyn",
    "Danielle", "Beverly", "Isabella", "Theresa", "Diana", "Natalie", "Brittany",
    "Charlotte", "Marie", "Kayla", "Alexis", "Lori",
]

LAST_NAMES = [
    "Smith", "Johnson", "Williams", "Brown", "Jones", "Garcia", "Miller", "Davis",
    "Rodriguez", "Martinez", "Hernandez", "Lopez", "Gonzalez", "Wilson", "Anderson",
    "Thomas", "Taylor", "Moore", "Jackson", "Martin", "Lee", "Perez", "Thompson",
    "White", "Harris", "Sanchez", "Clark", "Ramirez", "Lewis", "Robinson",
    "Walker", "Young", "Allen", "King", "Wright", "Scott", "Torres", "Nguyen",
    "Hill", "Flores", "Green", "Adams", "Nelson", "Baker", "Hall", "Rivera",
    "Campbell", "Mitchell", "Carter", "Roberts", "Gomez", "Phillips", "Evans",
    "Turner", "Diaz", "Parker", "Cruz", "Edwards", "Collins", "Reyes", "Stewart",
    "Morris", "Morales", "Murphy", "Cook", "Rogers", "Gutierrez", "Ortiz", "Morgan",
    "Cooper", "Peterson", "Bailey", "Reed", "Kelly", "Howard", "Ramos", "Kim",
    "Cox", "Ward", "Richardson", "Watson", "Brooks", "Chavez", "Wood", "James",
    "Bennett", "Gray", "Mendoza", "Ruiz", "Hughes", "Price", "Alvarez", "Castillo",
    "Sanders", "Patel", "Myers", "Long", "Ross", "Foster",
]

CITIES = [
    "New York", "Los Angeles", "Chicago", "Houston", "Phoenix", "Philadelphia",
    "San Antonio", "San Diego", "Dallas", "Austin", "San Jose", "Jacksonville",
    "Fort Worth", "Columbus", "Charlotte", "Indianapolis", "San Francisco",
    "Seattle", "Denver", "Nashville", "Oklahoma City", "Portland", "Las Vegas",
    "Memphis", "Louisville", "Baltimore", "Milwaukee", "Albuquerque", "Tucson",
    "Fresno", "Sacramento", "Mesa", "Atlanta", "Omaha", "Raleigh",
]

STREET_FORMATS = [
    "{number} {name} Street", "{number} {name} Avenue",
    "{number} {name} Boulevard", "{number} {name} Drive",
    "{number} {name} Lane", "{number} {name} Way",
    "{number} {name} Court", "{number} {name} Place",
]

POSTAL_CODE_FORMAT = "#####"
PHONE_FORMAT = "+1 (###) ###-####"
CURRENCY = "USD"
CURRENCY_SYMBOL = "$"

BANKS = [
    "JPMorgan Chase", "Bank of America", "Wells Fargo", "Citibank",
    "Goldman Sachs", "US Bank", "PNC Bank", "Capital One",
    "TD Bank", "Truist Financial", "Charles Schwab", "HSBC USA",
]

EMAIL_DOMAINS = ["gmail.com", "yahoo.com", "outlook.com", "icloud.com", "aol.com", "hotmail.com"]
COMPANY_SUFFIXES = ["Inc.", "LLC", "Corp.", "Co.", "Group", "Holdings", "Enterprises"]
COUNTRY_NAME = "United States"
NAME_ORDER = "given_family"



"""France locale data — fr_FR"""

FIRST_NAMES_MALE = [
    "Gabriel", "Léo", "Raphaël", "Arthur", "Louis", "Lucas", "Adam", "Jules", "Hugo",
    "Maël", "Liam", "Nathan", "Ethan", "Paul", "Noah", "Tom", "Sacha", "Théo", "Mathis",
    "Valentin", "Maxime", "Alexandre", "Antoine", "Nicolas", "Julien", "Thomas", "Pierre",
    "François", "Jean", "Jacques", "Philippe", "Michel", "Patrick", "Christophe", "Laurent",
    "Frédéric", "Olivier", "Stéphane", "Sébastien", "Éric", "Pascal", "Gérard", "Alain",
    "Bernard", "Daniel", "René", "Roger", "Marcel", "André", "Raymond", "Henri", "Claude",
    "Georges", "Yves", "Maurice", "Serge", "Didier", "Dominique", "Thierry", "Bruno",
    "Fabrice", "Sylvain", "Cédric", "Romain", "Florian", "Jérôme", "Guillaume", "Vincent",
    "Clément", "Adrien", "Bastien", "Quentin", "Axel", "Baptiste", "Enzo", "Nolan",
    "Robin", "Mattéo", "Gabin", "Émile", "Oscar", "Victor", "Félix", "Augustin", "Léon",
    "Timothée", "Marius", "Gustave", "Édouard", "Charles", "Martin", "Simon", "Damien",
    "Rémi", "Arnaud", "Ludovic", "Benoit", "Joël", "Gilles", "Yann", "Xavier", "Marc",
]

FIRST_NAMES_FEMALE = [
    "Emma", "Jade", "Louise", "Alice", "Ambre", "Léa", "Rose", "Anna", "Lina", "Mila",
    "Chloé", "Léonie", "Manon", "Juliette", "Julia", "Inès", "Zoé", "Charlotte", "Margaux",
    "Clara", "Camille", "Mathilde", "Sarah", "Marie", "Nathalie", "Isabelle", "Catherine",
    "Sylvie", "Véronique", "Françoise", "Monique", "Nicole", "Martine", "Christine",
    "Brigitte", "Denise", "Jacqueline", "Simone", "Madeleine", "Jeanne", "Paulette",
    "Yvette", "Colette", "Odette", "Germaine", "Suzanne", "Marguerite", "Lucienne",
    "Élise", "Anaïs", "Pauline", "Marine", "Aurélie", "Émilie", "Céline", "Élodie",
    "Stéphanie", "Delphine", "Sandrine", "Valérie", "Laurence", "Florence", "Béatrice",
    "Hélène", "Agnès", "Sophie", "Caroline", "Marion", "Audrey", "Laetitia", "Virginie",
    "Mélanie", "Laura", "Morgane", "Justine", "Océane", "Alexandra", "Lucie", "Clémence",
    "Agathe", "Victoria", "Lola", "Capucine", "Adèle", "Éléonore", "Apolline", "Constance",
    "Diane", "Gabrielle", "Margot", "Romane", "Salomé", "Anouk", "Célia", "Maëlle",
    "Nadia", "Fatima", "Yasmine", "Soraya",
]

LAST_NAMES = [
    "Martin", "Bernard", "Dubois", "Thomas", "Robert", "Richard", "Petit", "Durand",
    "Leroy", "Moreau", "Simon", "Laurent", "Lefebvre", "Michel", "Garcia", "David",
    "Bertrand", "Roux", "Vincent", "Fournier", "Morel", "Girard", "André", "Lefèvre",
    "Mercier", "Dupont", "Lambert", "Bonnet", "François", "Martinez", "Legrand", "Garnier",
    "Faure", "Rousseau", "Blanc", "Guérin", "Muller", "Henry", "Roussel", "Nicolas",
    "Perrin", "Morin", "Mathieu", "Clément", "Gauthier", "Dumont", "Lopez", "Fontaine",
    "Chevalier", "Robin", "Masson", "Sanchez", "Gerard", "Nguyen", "Boyer", "Denis",
    "Lemaire", "Duval", "Joly", "Gautier", "Roger", "Roche", "Roy", "Noël", "Meyer",
    "Lucas", "Meunier", "Jean", "Perez", "Marchand", "Dufour", "Blanchard", "Marie",
    "Barbier", "Brun", "Dumas", "Brunet", "Schmitt", "Leroux", "Colin", "Fernandez",
    "Pierre", "Renard", "Arnaud", "Rolland", "Caron", "Aubert", "Giraud", "Leclerc",
    "Vidal", "Bourgeois", "Renaud", "Lemoine", "Picard", "Gaillard", "Philippe", "Leclercq",
    "Lacroix", "Fabre", "Dupuis", "Olivier",
]

CITIES = [
    "Paris", "Marseille", "Lyon", "Toulouse", "Nice", "Nantes", "Montpellier", "Strasbourg",
    "Bordeaux", "Lille", "Rennes", "Grenoble", "Rouen", "Toulon", "Dijon", "Angers",
    "Nîmes", "Clermont-Ferrand", "Le Havre", "Aix-en-Provence", "Brest", "Tours", "Amiens",
    "Limoges", "Metz", "Perpignan", "Besançon", "Orléans", "Caen", "Saint-Étienne",
]

STREET_FORMATS = [
    "{number} Rue {name}", "{number} Avenue {name}", "{number} Boulevard {name}",
    "{number} Place {name}", "{number} Allée {name}", "{number} Impasse {name}",
]

POSTAL_CODE_FORMAT = "#####"
PHONE_FORMAT = "+33 # ## ## ## ##"
CURRENCY = "EUR"
CURRENCY_SYMBOL = "€"

BANKS = [
    "BNP Paribas", "Crédit Agricole", "Société Générale", "Banque Populaire",
    "Crédit Mutuel", "La Banque Postale", "LCL", "Caisse d'Épargne", "CIC",
    "HSBC France", "AXA Banque", "Boursorama",
]

EMAIL_DOMAINS = ["gmail.com", "orange.fr", "free.fr", "sfr.fr", "laposte.net", "yahoo.fr"]
COMPANY_SUFFIXES = ["SA", "SAS", "SARL", "EURL", "SCI", "& Fils", "& Cie"]
COUNTRY_NAME = "France"
NAME_ORDER = "given_family"



"""
fabrikate — Contextually coherent mock data generation.

Unlike random fake data, fabrikate generates data that makes sense together.
A person in Tokyo gets a Japanese name, address, phone format, and bank.
A company's employees, products, and transactions are internally consistent.
"""

__version__ = "0.3.0"

from fabrikate.context import Context
from fabrikate.world import World
from fabrikate.generators.person import Person, Address, Education, Vehicle, MedicalRecord
from fabrikate.generators.company import Company
from fabrikate.generators.transaction import Transaction
from fabrikate.generators.product import Product
from fabrikate.locales import supported_locales

__all__ = [
    "Context",
    "World",
    "Person",
    "Address",
    "Education",
    "Vehicle",
    "MedicalRecord",
    "Company",
    "Transaction",
    "Product",
    "supported_locales",
]



"""
Company generator — creates companies with coherent employees,
products, and financial context.

A company inherits its locale context and passes it down to
any employees or products generated within it.
"""

from __future__ import annotations

import uuid
from dataclasses import dataclass, field
from typing import Optional

from fabrikate.context import Context
from fabrikate.locales import get_locale
from fabrikate.generators.person import Person
from fabrikate.distributions import log_normal_float
from fabrikate.reference_data import TAX_ID_FORMATS, DEPARTMENTS


INDUSTRY_KEYWORDS = {
    "tech": ["Digital", "Cloud", "Data", "Cyber", "Net", "Quantum", "AI", "Logic"],
    "finance": ["Capital", "Trust", "Equity", "Asset", "Wealth", "Venture", "Harbor"],
    "health": ["Health", "Vita", "Med", "Bio", "Care", "Life", "Pharma", "Thera"],
    "retail": ["Market", "Store", "Shop", "Goods", "Trade", "Fresh", "Home"],
    "food": ["Fresh", "Harvest", "Kitchen", "Taste", "Green", "Farm", "Spice"],
    "manufacturing": ["Steel", "Forge", "Iron", "Works", "Craft", "Build", "Core"],
}

GENERIC_KEYWORDS = ["Global", "Prime", "Summit", "Atlas", "Apex", "Nova", "Arc"]


@dataclass
class Company:
    """
    A contextually coherent company.

    The company's name style, suffix, currency, and bank are locale-appropriate.
    Employees generated via `hire()` inherit the company's context.
    """

    id: str = ""
    name: str = ""
    industry: str = ""
    founded_year: int = 0
    employee_count: int = 0
    currency: str = ""
    bank: str = ""
    city: str = ""
    country: str = ""
    tax_id: str = ""
    annual_revenue: float = 0.0
    website: str = ""
    departments: list[str] = field(default_factory=list)
    employees: list[Person] = field(default_factory=list)
    _ctx: Optional[Context] = field(default=None, repr=False)

    @classmethod
    def generate(cls, ctx: Context, **overrides) -> Company:
        """
        Generate a coherent company within the given context.

        Parameters
        ----------
        ctx : Context
            The generation context.
        **overrides
            Override any field: industry="tech", name="Acme Corp", etc.
        """
        locale = get_locale(ctx.locale)
        rng = ctx._rng

        company = cls(_ctx=ctx)
        company.id = overrides.get("id", str(uuid.UUID(int=rng.getrandbits(128))))

        # Industry
        company.industry = overrides.get("industry", ctx.industry or rng.choice(
            list(INDUSTRY_KEYWORDS.keys())
        ))

        # Company name — industry-appropriate + locale suffix
        keywords = INDUSTRY_KEYWORDS.get(company.industry, GENERIC_KEYWORDS)
        word1 = rng.choice(keywords)
        word2 = rng.choice(GENERIC_KEYWORDS)
        suffix = rng.choice(locale.company_suffixes)
        company.name = overrides.get("name", f"{word1}{word2} {suffix}")

        # Location
        company.city = overrides.get("city", rng.choice(locale.cities))
        company.country = locale.country_name

        # Financials
        company.currency = locale.currency
        company.bank = overrides.get("bank", rng.choice(locale.banks))
        company.founded_year = overrides.get(
            "founded_year", rng.randint(1970, ctx.year - 1)
        )
        company.employee_count = overrides.get(
            "employee_count", rng.choice([5, 12, 25, 50, 100, 250, 500, 1000])
        )

        # Tax ID — locale format
        cc = ctx.country_code
        tax_info = TAX_ID_FORMATS.get(cc, {"name": "Tax ID", "format": "##########"})
        from fabrikate.generators.person import _format_id
        company.tax_id = overrides.get("tax_id", _format_id(tax_info["format"], rng))

        # Annual revenue (log-normal, scales with employee count)
        base_revenue = company.employee_count * rng.randint(80000, 300000)
        company.annual_revenue = overrides.get("annual_revenue", round(log_normal_float(rng, base_revenue, sigma=0.4), -3))

        # Website
        name_slug = company.name.split()[0].lower().replace(".", "").replace(",", "")
        company.website = overrides.get("website", f"https://www.{name_slug}.com")

        # Departments (scale with company size)
        num_depts = min(len(DEPARTMENTS), max(2, company.employee_count // 20))
        company.departments = overrides.get("departments", rng.sample(DEPARTMENTS, num_depts))

        ctx.register(company)
        return company

    def hire(self, count: int = 1, **person_overrides) -> list[Person]:
        """
        Generate employees that belong to this company.

        Employees inherit the company's context (locale, industry, etc.)
        and are linked to this company by ID.
        """
        new_employees = []
        for _ in range(count):
            person = Person.generate(
                self._ctx,
                company_id=self.id,
                **person_overrides,
            )
            new_employees.append(person)
            self.employees.append(person)
        return new_employees

    def to_dict(self) -> dict:
        """Export as a plain dictionary."""
        return {
            "id": self.id,
            "name": self.name,
            "industry": self.industry,
            "founded_year": self.founded_year,
            "employee_count": self.employee_count,
            "currency": self.currency,
            "bank": self.bank,
            "city": self.city,
            "country": self.country,
            "tax_id": self.tax_id,
            "annual_revenue": self.annual_revenue,
            "website": self.website,
            "departments": self.departments,
            "employees": [e.to_dict() for e in self.employees],
        }



"""
Weighted random utilities for realistic data distributions.

Instead of uniform random.choice, these functions pick values
with real-world frequency distributions — some names are more
common, most transactions are small, ages follow bell curves.
"""

from __future__ import annotations

import math
import random
from typing import TypeVar

T = TypeVar("T")


def weighted_choice(rng: random.Random, items: list[T], weights: list[float]) -> T:
    """Pick an item with the given frequency weights."""
    total = sum(weights)
    r = rng.uniform(0, total)
    cumulative = 0.0
    for item, weight in zip(items, weights):
        cumulative += weight
        if r <= cumulative:
            return item
    return items[-1]


def bell_curve_int(rng: random.Random, low: int, high: int, mean: float | None = None, sd: float | None = None) -> int:
    """Generate an integer from a bell curve (clamped normal distribution)."""
    if mean is None:
        mean = (low + high) / 2
    if sd is None:
        sd = (high - low) / 6  # 99.7% within range
    value = rng.gauss(mean, sd)
    return max(low, min(high, int(round(value))))


def log_normal_float(rng: random.Random, median: float, sigma: float = 0.8, low: float = 0.01, high: float | None = None) -> float:
    """
    Generate a log-normal value — many small, few large.
    Good for transaction amounts, salaries, revenue.
    """
    mu = math.log(median)
    value = math.exp(rng.gauss(mu, sigma))
    value = max(low, value)
    if high is not None:
        value = min(high, value)
    return round(value, 2)


def skewed_choice(rng: random.Random, items: list[T], skew: float = 1.5) -> T:
    """
    Pick from a list where earlier items are more common.
    skew=1.0 is uniform, skew=2.0 heavily favors the first items.
    Good for name frequency (most common names listed first).
    """
    n = len(items)
    weights = [(n - i) ** skew for i in range(n)]
    return weighted_choice(rng, items, weights)


# --- Locale-specific distributions ---

# Blood type distributions by country (approximate percentages)
BLOOD_TYPE_DIST: dict[str, dict[str, float]] = {
    "US": {"O+": 37.4, "A+": 35.7, "B+": 8.5, "AB+": 3.4, "O-": 6.6, "A-": 6.3, "B-": 1.5, "AB-": 0.6},
    "JP": {"A+": 40.0, "O+": 30.0, "B+": 20.0, "AB+": 10.0, "A-": 0.0, "O-": 0.0, "B-": 0.0, "AB-": 0.0},
    "DE": {"A+": 37.0, "O+": 35.0, "B+": 9.0, "AB+": 4.0, "A-": 6.0, "O-": 6.0, "B-": 2.0, "AB-": 1.0},
    "BR": {"O+": 36.0, "A+": 34.0, "B+": 8.0, "AB+": 2.5, "O-": 9.0, "A-": 8.0, "B-": 2.0, "AB-": 0.5},
    "IN": {"O+": 36.5, "B+": 32.1, "A+": 22.9, "AB+": 6.4, "O-": 2.0, "A-": 0.5, "B-": 0.3, "AB-": 0.1},
    "GB": {"O+": 35.0, "A+": 30.0, "B+": 8.0, "AB+": 2.0, "O-": 13.0, "A-": 8.0, "B-": 2.0, "AB-": 1.0},
    "FR": {"A+": 38.0, "O+": 36.0, "B+": 7.5, "AB+": 3.0, "A-": 7.0, "O-": 6.0, "B-": 1.5, "AB-": 1.0},
    "KR": {"A+": 34.0, "O+": 28.0, "B+": 27.0, "AB+": 11.0, "A-": 0.0, "O-": 0.0, "B-": 0.0, "AB-": 0.0},
    "ES": {"O+": 36.0, "A+": 34.0, "B+": 8.0, "AB+": 2.5, "O-": 9.0, "A-": 7.0, "B-": 2.0, "AB-": 0.5},
    "SA": {"O+": 48.0, "A+": 24.0, "B+": 17.0, "AB+": 4.0, "O-": 4.0, "A-": 2.0, "B-": 1.0, "AB-": 0.0},
    "NG": {"O+": 51.0, "A+": 21.0, "B+": 21.0, "AB+": 3.0, "O-": 2.0, "A-": 1.0, "B-": 1.0, "AB-": 0.0},
}

# Average children per family by country
CHILDREN_DIST: dict[str, dict[int, float]] = {
    "US": {0: 15, 1: 20, 2: 35, 3: 20, 4: 7, 5: 3},
    "JP": {0: 30, 1: 35, 2: 25, 3: 8, 4: 2, 5: 0},
    "DE": {0: 25, 1: 30, 2: 30, 3: 10, 4: 4, 5: 1},
    "BR": {0: 12, 1: 20, 2: 30, 3: 22, 4: 10, 5: 6},
    "IN": {0: 8, 1: 15, 2: 35, 3: 25, 4: 12, 5: 5},
    "GB": {0: 18, 1: 25, 2: 35, 3: 15, 4: 5, 5: 2},
    "FR": {0: 15, 1: 20, 2: 35, 3: 20, 4: 7, 5: 3},
    "KR": {0: 35, 1: 35, 2: 22, 3: 6, 4: 2, 5: 0},
    "ES": {0: 20, 1: 30, 2: 35, 3: 10, 4: 4, 5: 1},
    "SA": {0: 5, 1: 8, 2: 15, 3: 25, 4: 25, 5: 22},
    "NG": {0: 3, 1: 5, 2: 10, 3: 20, 4: 30, 5: 32},
}

# Marital status by age bracket (generic, locale can override)
def marital_status(rng: random.Random, age: int) -> str:
    if age < 22:
        return weighted_choice(rng, ["single", "single", "married", "single"], [80, 5, 10, 5])
    elif age < 30:
        return weighted_choice(rng, ["single", "married", "engaged", "divorced"], [45, 40, 10, 5])
    elif age < 45:
        return weighted_choice(rng, ["married", "single", "divorced", "widowed"], [55, 20, 20, 5])
    elif age < 60:
        return weighted_choice(rng, ["married", "divorced", "widowed", "single"], [50, 25, 15, 10])
    else:
        return weighted_choice(rng, ["married", "widowed", "divorced", "single"], [40, 30, 20, 10])



"""Spain locale data — es_ES"""

FIRST_NAMES_MALE = [
    "Hugo", "Mateo", "Martín", "Lucas", "Leo", "Daniel", "Alejandro", "Pablo", "Manuel",
    "Álvaro", "Adrián", "David", "Mario", "Diego", "Javier", "Carlos", "Miguel", "Sergio",
    "Antonio", "Iker", "Marcos", "Gonzalo", "Iván", "Raúl", "Rubén", "Ángel", "Fernando",
    "Jorge", "Alberto", "Víctor", "Pedro", "Francisco", "José", "Juan", "Rafael", "Andrés",
    "Enrique", "Luis", "Tomás", "Ignacio", "Guillermo", "Nicolás", "Samuel", "Héctor",
    "Emilio", "Ramón", "Arturo", "Salvador", "Eduardo", "Ricardo", "Cristian", "Germán",
    "Félix", "Lorenzo", "Omar", "Bruno", "Darío", "Jaime", "Xavier", "Agustín",
    "Aitor", "Ariel", "Bautista", "Borja", "Claudio", "Damián", "Esteban", "Fabián",
    "Gaspar", "Gregorio", "Hernán", "Ismael", "Joaquín", "Kevin", "Lázaro", "Leandro",
    "Matías", "Néstor", "Orlando", "Patricio", "Rodrigo", "Santiago", "Sebastián", "Simón",
    "Teodoro", "Valentín", "Wenceslao", "Yeray", "Alonso", "Benito", "César", "Domingo",
    "Efrén", "Fidel", "Gerardo", "Hilario", "Julio", "Leonardo", "Mariano", "Pascual",
]

FIRST_NAMES_FEMALE = [
    "Lucía", "Sofía", "Martina", "María", "Julia", "Paula", "Emma", "Daniela", "Valeria",
    "Alba", "Carmen", "Sara", "Noa", "Claudia", "Irene", "Elena", "Laura", "Marta",
    "Laia", "Isabel", "Natalia", "Ana", "Andrea", "Cristina", "Beatriz", "Teresa",
    "Rosa", "Pilar", "Dolores", "Mercedes", "Francisca", "Concepción", "Josefa", "Manuela",
    "Antonia", "Encarnación", "Rosario", "Amparo", "Inmaculada", "Soledad", "Raquel",
    "Silvia", "Patricia", "Verónica", "Mónica", "Nuria", "Alicia", "Eva", "Rocío",
    "Esther", "Lorena", "Miriam", "Sonia", "Susana", "Yolanda", "Montserrat", "Begoña",
    "Virginia", "Carolina", "Adriana", "Inés", "Victoria", "Lidia", "Ángela", "Diana",
    "Olga", "Sandra", "Celia", "Marina", "Blanca", "Noelia", "Ainhoa", "Nerea",
    "Ariadna", "Carlota", "Abril", "Jimena", "Ainara", "Candela", "Carla", "Vera",
    "Leyre", "Olivia", "Aitana", "Alejandra", "Chloe", "Lola", "Luna", "Manuela",
    "Valentina", "Ariana", "Ximena", "Esperanza", "Gloria", "Leonor", "Paloma", "Paz",
    "Remedios", "Consuelo",
]

LAST_NAMES = [
    "García", "Rodríguez", "Martínez", "López", "González", "Hernández", "Pérez", "Sánchez",
    "Ramírez", "Torres", "Flores", "Rivera", "Gómez", "Díaz", "Moreno", "Muñoz", "Álvarez",
    "Romero", "Navarro", "Ruiz", "Jiménez", "Domínguez", "Vázquez", "Ramos", "Gil",
    "Serrano", "Blanco", "Molina", "Suárez", "Ortega", "Delgado", "Castro", "Ortiz",
    "Rubio", "Marín", "Sanz", "Iglesias", "Núñez", "Medina", "Garrido", "Cortés",
    "Castillo", "Santos", "Guerrero", "Lozano", "Cano", "Prieto", "Méndez", "Cruz",
    "Calvo", "Gallego", "Herrera", "Peña", "León", "Márquez", "Cabrera", "Vidal",
    "Lorenzo", "Campos", "Reyes", "Vega", "Fuentes", "Carrasco", "Diez", "Caballero",
    "Nieto", "Aguilar", "Pascual", "Herrero", "Montero", "Giménez", "Hidalgo", "Ibáñez",
    "Soler", "Vargas", "Ferrer", "Arias", "Rojas", "Mora", "Crespo", "Parra",
    "Esteban", "Bravo", "Gallardo", "Duarte", "Benítez", "Moya", "Salas", "Velasco",
    "Palacios", "Rivas", "Santana", "Bermúdez", "Cordero", "Figueroa", "Acosta", "Valencia",
]

CITIES = [
    "Madrid", "Barcelona", "Valencia", "Sevilla", "Zaragoza", "Málaga", "Murcia", "Palma",
    "Bilbao", "Alicante", "Córdoba", "Valladolid", "Granada", "Oviedo", "Salamanca",
    "Vigo", "Gijón", "Vitoria", "La Coruña", "Elche", "Santa Cruz de Tenerife",
    "Las Palmas", "Pamplona", "Almería", "Burgos", "Santander", "Castellón", "Albacete",
    "San Sebastián", "Logroño",
]

STREET_FORMATS = [
    "Calle {name}, {number}", "Avenida de {name}, {number}",
    "Paseo de {name}, {number}", "Plaza de {name}, {number}",
    "Carrera de {name}, {number}", "Camino de {name}, {number}",
]

POSTAL_CODE_FORMAT = "#####"
PHONE_FORMAT = "+34 ### ### ###"
CURRENCY = "EUR"
CURRENCY_SYMBOL = "€"

BANKS = [
    "Santander", "BBVA", "CaixaBank", "Bankinter", "Sabadell", "Unicaja",
    "Kutxabank", "Ibercaja", "Abanca", "Bankia", "Openbank", "ING España",
]

EMAIL_DOMAINS = ["gmail.com", "hotmail.es", "yahoo.es", "outlook.es", "telefonica.net", "terra.es"]
COMPANY_SUFFIXES = ["S.A.", "S.L.", "S.L.U.", "S.Coop.", "& Cía.", "Hermanos"]
COUNTRY_NAME = "España"
NAME_ORDER = "given_family"



"""
Context holds the shared state that keeps generated data coherent.

When you create a person in a context set to "Tokyo", everything about
that person — name, address, phone, bank — will be Japanese. When you
create a company, its employees inherit the company's context.
"""

from __future__ import annotations

import random
from dataclasses import dataclass, field
from typing import Optional


@dataclass
class Context:
    """
    A generation context that ensures coherent mock data.

    Parameters
    ----------
    locale : str
        A locale code like "ja_JP", "en_US", "de_DE", "pt_BR".
        Controls names, addresses, phone formats, currencies, etc.
    seed : int or None
        Random seed for reproducibility. Same seed = same data every time.
    year : int
        The reference year for generating dates and ages.
    industry : str or None
        Industry context for companies and products (e.g., "tech", "finance").
    """

    locale: str = "en_US"
    seed: Optional[int] = None
    year: int = 2025
    industry: Optional[str] = None
    _rng: random.Random = field(default_factory=random.Random, repr=False)
    _entity_registry: list = field(default_factory=list, repr=False)

    def __post_init__(self):
        if self.seed is not None:
            self._rng.seed(self.seed)

    @property
    def country_code(self) -> str:
        """Extract country code from locale, e.g. 'ja_JP' -> 'JP'."""
        if "_" in self.locale:
            return self.locale.split("_")[1]
        return self.locale.upper()

    @property
    def language_code(self) -> str:
        """Extract language code from locale, e.g. 'ja_JP' -> 'ja'."""
        if "_" in self.locale:
            return self.locale.split("_")[0]
        return self.locale.lower()

    def child(self, **overrides) -> Context:
        """
        Create a child context that inherits settings but can override them.

        Useful when a company in Tokyo has a branch in New York — the branch
        gets a child context with locale="en_US" but shares the same seed
        progression.
        """
        params = {
            "locale": self.locale,
            "seed": None,  # child uses parent's rng progression
            "year": self.year,
            "industry": self.industry,
        }
        params.update(overrides)
        child_ctx = Context(**params)
        child_ctx._rng = self._rng  # share RNG for determinism
        return child_ctx

    def register(self, entity) -> None:
        """Register an entity so other generators can reference it."""
        self._entity_registry.append(entity)

    def find(self, entity_type: type, **filters) -> list:
        """
        Find registered entities by type and optional attribute filters.

        Example: ctx.find(Person, company_id=company.id)
        """
        results = [e for e in self._entity_registry if isinstance(e, entity_type)]
        for key, value in filters.items():
            results = [e for e in results if getattr(e, key, None) == value]
        return results



"""Saudi Arabia locale data — ar_SA"""

FIRST_NAMES_MALE = [
    "محمد", "عبدالله", "عبدالعزيز", "فهد", "سلطان", "خالد", "سعود", "نايف", "تركي",
    "بندر", "عبدالرحمن", "سعد", "ماجد", "فيصل", "أحمد", "إبراهيم", "عمر", "علي",
    "ياسر", "طارق", "حسن", "حسين", "يوسف", "صالح", "عادل", "مشاري", "وليد",
    "مساعد", "ناصر", "راشد", "سالم", "زياد", "مازن", "أنس", "بدر", "هاني",
    "عماد", "كريم", "جمال", "منصور", "مروان", "باسل", "رائد", "سامي", "عصام",
    "نواف", "ثامر", "مشعل", "متعب", "عبدالمجيد", "عبدالملك", "عبدالكريم", "عبداللطيف",
    "عبدالإله", "عبدالمحسن", "عبدالهادي", "عبدالواحد", "عبدالرقيب", "عبدالسلام",
    "عبدالقادر", "لؤي", "حمد", "طلال", "هشام", "رامي", "أسامة", "معاذ", "أيمن",
    "شريف", "حاتم", "فواز", "عايض", "صقر", "دخيل", "جابر", "سليمان", "عيسى",
    "موسى", "داود", "نوح", "آدم", "إلياس", "لقمان", "شعيب", "هود", "صلاح",
    "رضوان", "عثمان", "أبوبكر", "زيد", "حمزة", "بلال", "أمجد", "عاصم", "وائل",
    "غسان",
]

FIRST_NAMES_FEMALE = [
    "نورة", "سارة", "لمى", "ريم", "هيفاء", "مها", "أمل", "دلال", "منال", "غادة",
    "فاطمة", "عائشة", "خديجة", "مريم", "زينب", "هند", "أسماء", "سمر", "رنا",
    "لينا", "هدى", "سلوى", "نجلاء", "ابتسام", "نادية", "سميرة", "حنان", "وفاء",
    "صفاء", "رجاء", "بشرى", "زهرة", "ياسمين", "جمانة", "رهف", "لجين", "تالا",
    "دانة", "جوري", "ريناد", "شهد", "وجدان", "حور", "سدين", "غلا", "ميادة",
    "بيان", "أريج", "عبير", "نسرين", "رنيم", "ملاك", "جنى", "سلمى", "يارا",
    "ديمة", "رزان", "هيا", "الجوهرة", "العنود", "لطيفة", "بدور", "شيخة", "موضي",
    "هناء", "إيمان", "آلاء", "روان", "لمياء", "مي", "ندى", "رباب", "سحر",
    "فوزية", "حصة", "منيرة", "جواهر", "أميرة", "شيماء", "مرام", "بسمة", "نجوى",
    "علياء", "خلود", "لولوة", "مشاعل", "عزيزة", "ريحانة", "سندس", "إسراء", "كوثر",
    "رقية", "سكينة", "فريدة", "حبيبة", "تهاني", "ليلى", "عفاف", "نعيمة", "راوية",
]

LAST_NAMES = [
    "القحطاني", "العتيبي", "الغامدي", "الدوسري", "الشهري", "الحربي", "الزهراني",
    "المطيري", "الشمري", "العنزي", "السبيعي", "المالكي", "الأحمدي", "البلوي",
    "الرشيدي", "الجهني", "الحارثي", "السلمي", "الثبيتي", "البقمي", "العمري",
    "الصاعدي", "الشريف", "الهاشمي", "الحسيني", "العلوي", "الزيدي", "المحمدي",
    "الخالدي", "السعدي", "الناصري", "الفهدي", "التميمي", "القرشي", "المكي",
    "المدني", "الأنصاري", "العمراني", "الشيباني", "الكندي", "الراجحي", "الدخيل",
    "الشويعر", "النمر", "البراك", "الفوزان", "السويلم", "المنيف", "الحقباني",
    "العسيري", "الخثعمي", "الوادعي", "النخلي", "الغزالي", "العسكري", "الرومي",
    "الفارسي", "الهندي", "التركي", "اليماني", "المصري", "الشامي", "العراقي",
    "الحجازي", "النجدي", "القصيمي", "الحساوي", "الدمشقي", "البغدادي", "الموصلي",
    "السديري", "الثنيان", "الجفالي", "العليان", "الراشد", "المحيسن", "الطريقي",
    "البكري", "الماضي", "النويصر", "الحمدان", "السيف", "الرميح", "الحبيب",
    "العقيل", "الشثري", "المنقور", "اللحيدان", "العيسى", "البسام", "المقبل",
    "الزامل", "القاسم", "الربيعة", "السالم", "العبيد", "الصقر", "الحميد", "النفيسة",
]

CITIES = [
    "الرياض", "جدة", "مكة", "المدينة", "الدمام", "الخبر", "تبوك", "أبها", "الطائف",
    "بريدة", "خميس مشيط", "حائل", "نجران", "جازان", "ينبع", "القطيف", "الجبيل",
    "الظهران", "عرعر", "سكاكا", "الباحة", "بيشة", "الخرج", "عنيزة", "الرس",
]

STREET_FORMATS = [
    "شارع {name}، حي {district}، {number}",
    "طريق {name}، {number}",
]

POSTAL_CODE_FORMAT = "#####"
PHONE_FORMAT = "+966 ## ### ####"
CURRENCY = "SAR"
CURRENCY_SYMBOL = "﷼"

BANKS = [
    "البنك الأهلي السعودي", "بنك الراجحي", "بنك الرياض", "البنك السعودي البريطاني",
    "بنك الجزيرة", "البنك السعودي الفرنسي", "بنك البلاد", "بنك الإنماء",
    "البنك العربي الوطني", "بنك الاستثمار السعودي", "بنك السعودي الهولندي",
    "مصرف الراجحي",
]

EMAIL_DOMAINS = ["gmail.com", "hotmail.com", "outlook.sa", "yahoo.com", "outlook.com", "stc.com.sa"]
COMPANY_SUFFIXES = ["ش.م.", "ش.ذ.م.م", "مؤسسة", "شركة", "مجموعة"]
COUNTRY_NAME = "المملكة العربية السعودية"
NAME_ORDER = "given_family"



"""
Person generator — creates people that make sense in their locale.

v0.3.0: Added DOB, job title, salary, national ID, marital status,
blood type, height, weight, phone carrier, credit card type, username,
education, and vehicle. All locale-aware with realistic distributions.
"""

from __future__ import annotations

import hashlib
import uuid
from dataclasses import dataclass, field
from datetime import date
from typing import Optional

from fabrikate.context import Context
from fabrikate.locales import get_locale
from fabrikate.distributions import (
    bell_curve_int, log_normal_float, weighted_choice, skewed_choice, marital_status,
    BLOOD_TYPE_DIST, CHILDREN_DIST,
)
from fabrikate.reference_data import (
    JOB_TITLES, SALARY_RANGES, NATIONAL_ID_FORMATS, PAYMENT_METHODS,
    PHONE_CARRIERS, CREDIT_CARD_TYPES, UNIVERSITIES, DEGREE_TYPES,
    DEGREE_FIELDS, CAR_BRANDS, INSURANCE_PROVIDERS, BIOMETRICS,
)


@dataclass
class Address:
    street: str
    city: str
    postal_code: str
    country: str

    def __str__(self) -> str:
        return f"{self.street}, {self.city} {self.postal_code}, {self.country}"

    def to_dict(self) -> dict:
        return {"street": self.street, "city": self.city, "postal_code": self.postal_code, "country": self.country}


@dataclass
class Education:
    university: str = ""
    degree: str = ""
    field_of_study: str = ""
    graduation_year: int = 0

    def to_dict(self) -> dict:
        return {"university": self.university, "degree": self.degree, "field_of_study": self.field_of_study, "graduation_year": self.graduation_year}


@dataclass
class Vehicle:
    brand: str = ""
    year: int = 0
    plate_number: str = ""

    def to_dict(self) -> dict:
        return {"brand": self.brand, "year": self.year, "plate_number": self.plate_number}


@dataclass
class MedicalRecord:
    blood_type: str = ""
    height_cm: int = 0
    weight_kg: int = 0
    insurance_provider: str = ""

    def to_dict(self) -> dict:
        return {"blood_type": self.blood_type, "height_cm": self.height_cm, "weight_kg": self.weight_kg, "insurance_provider": self.insurance_provider}


@dataclass
class Person:
    """
    A contextually coherent person with full demographic detail.

    All attributes are consistent with the context's locale — names, address,
    phone, email, bank, salary, ID format, and more.
    """

    # Identity
    id: str = ""
    first_name: str = ""
    last_name: str = ""
    full_name: str = ""
    gender: str = ""
    age: int = 0
    date_of_birth: str = ""
    nationality: str = ""
    national_id: str = ""
    marital_status: str = ""
    children: int = 0

    # Contact
    email: str = ""
    phone: str = ""
    phone_carrier: str = ""
    username: str = ""
    address: Optional[Address] = None

    # Financial
    bank: str = ""
    credit_card_type: str = ""
    salary: float = 0.0
    salary_currency: str = ""

    # Employment
    job_title: str = ""
    department: str = ""
    company_id: Optional[str] = None

    # Education
    education: Optional[Education] = None

    # Vehicle
    vehicle: Optional[Vehicle] = None

    # Medical
    medical: Optional[MedicalRecord] = None

    _ctx: Optional[Context] = field(default=None, repr=False)

    @classmethod
    def generate(cls, ctx: Context, **overrides) -> Person:
        locale = get_locale(ctx.locale)
        rng = ctx._rng
        cc = ctx.country_code

        person = cls(_ctx=ctx)
        person.id = overrides.get("id", str(uuid.UUID(int=rng.getrandbits(128))))

        # --- Gender ---
        person.gender = overrides.get("gender", rng.choice(["male", "female"]))

        # --- Name (skewed — common names more likely) ---
        if person.gender == "male":
            person.first_name = overrides.get("first_name", skewed_choice(rng, locale.first_names_male, skew=1.3))
        else:
            person.first_name = overrides.get("first_name", skewed_choice(rng, locale.first_names_female, skew=1.3))
        person.last_name = overrides.get("last_name", skewed_choice(rng, locale.last_names, skew=1.4))

        if locale.name_order == "family_given":
            person.full_name = f"{person.last_name} {person.first_name}"
        else:
            person.full_name = f"{person.first_name} {person.last_name}"

        # --- Age (bell curve centered at 35) ---
        person.age = overrides.get("age", bell_curve_int(rng, 18, 70, mean=35, sd=12))

        # --- Date of birth ---
        birth_year = ctx.year - person.age
        birth_month = rng.randint(1, 12)
        birth_day = rng.randint(1, 28)
        person.date_of_birth = overrides.get("date_of_birth", f"{birth_year}-{birth_month:02d}-{birth_day:02d}")

        # --- Nationality ---
        person.nationality = overrides.get("nationality", locale.country_name)

        # --- National ID ---
        id_info = NATIONAL_ID_FORMATS.get(cc, {"name": "ID", "format": "############"})
        person.national_id = overrides.get("national_id", _format_id(id_info["format"], rng))

        # --- Marital status (age-dependent) ---
        person.marital_status = overrides.get("marital_status", marital_status(rng, person.age))

        # --- Children (locale-dependent distribution) ---
        children_dist = CHILDREN_DIST.get(cc, CHILDREN_DIST["US"])
        kids = list(children_dist.keys())
        kid_weights = list(children_dist.values())
        if person.age < 25:
            person.children = overrides.get("children", weighted_choice(rng, [0, 0, 1, 0], [80, 5, 10, 5]))
        else:
            person.children = overrides.get("children", weighted_choice(rng, kids, kid_weights))

        # --- Phone ---
        person.phone = _format_pattern(locale.phone_format, rng)

        # --- Phone carrier ---
        carriers = PHONE_CARRIERS.get(cc, ["Unknown"])
        person.phone_carrier = overrides.get("phone_carrier", rng.choice(carriers))

        # --- Address ---
        city = rng.choice(locale.cities)
        street_names = _get_street_names(cc)
        street_fmt = rng.choice(locale.street_formats)
        street = street_fmt.format(
            number=rng.randint(1, 9999),
            name=rng.choice(street_names),
            city=city,
            ward=rng.choice(_DISTRICTS.get(cc, {}).get("wards", ["Central"])),
            district=rng.choice(_DISTRICTS.get(cc, {}).get("districts", ["Central"])),
        )
        postal = _format_postal(locale.postal_code_format, rng)
        person.address = Address(street=street, city=city, postal_code=postal, country=locale.country_name)

        # --- Email & username ---
        name_slug = f"{_romanize(person.first_name)}.{_romanize(person.last_name)}".lower().replace(" ", "")
        domain = rng.choice(locale.email_domains)
        person.email = overrides.get("email", f"{name_slug}@{domain}")
        person.username = overrides.get("username", f"{name_slug}{rng.randint(1, 999)}")

        # --- Bank ---
        person.bank = overrides.get("bank", rng.choice(locale.banks))

        # --- Credit card type (locale-weighted) ---
        cc_types = CREDIT_CARD_TYPES.get(cc, CREDIT_CARD_TYPES["US"])
        cc_items = [t[0] for t in cc_types]
        cc_weights = [t[1] for t in cc_types]
        person.credit_card_type = overrides.get("credit_card_type", weighted_choice(rng, cc_items, cc_weights))

        # --- Salary (log-normal, locale-appropriate) ---
        sal_range = SALARY_RANGES.get(cc, (30000, 60000, 150000))
        person.salary = overrides.get("salary", log_normal_float(rng, sal_range[1], sigma=0.6, low=sal_range[0], high=sal_range[2]))
        person.salary_currency = locale.currency

        # --- Job title ---
        industry = ctx.industry or "general"
        titles = JOB_TITLES.get(industry, JOB_TITLES["general"])
        person.job_title = overrides.get("job_title", rng.choice(titles))

        # --- Department ---
        from fabrikate.reference_data import DEPARTMENTS
        person.department = overrides.get("department", rng.choice(DEPARTMENTS))

        # --- Company link ---
        person.company_id = overrides.get("company_id", None)

        # --- Education ---
        unis = UNIVERSITIES.get(cc, UNIVERSITIES.get("US", ["University"]))
        deg_items = [d[0] for d in DEGREE_TYPES]
        deg_weights = [d[1] for d in DEGREE_TYPES]
        degree = weighted_choice(rng, deg_items, deg_weights)
        grad_offset = {"Bachelor's": 22, "Master's": 24, "PhD": 28, "Associate's": 20, "Diploma": 19}
        grad_year = birth_year + grad_offset.get(degree, 22) + rng.randint(-1, 2)
        person.education = Education(
            university=rng.choice(unis),
            degree=degree,
            field_of_study=rng.choice(DEGREE_FIELDS),
            graduation_year=min(grad_year, ctx.year),
        )

        # --- Vehicle (70% chance of having one) ---
        if rng.random() < 0.70:
            car_data = CAR_BRANDS.get(cc, CAR_BRANDS["US"])
            brands = [c[0] for c in car_data]
            brand_weights = [c[1] for c in car_data]
            car_year = rng.randint(max(ctx.year - 15, 2005), ctx.year)
            plate = _format_plate(cc, rng)
            person.vehicle = Vehicle(
                brand=weighted_choice(rng, brands, brand_weights),
                year=car_year,
                plate_number=plate,
            )

        # --- Medical ---
        bt_dist = BLOOD_TYPE_DIST.get(cc, BLOOD_TYPE_DIST["US"])
        bt_items = list(bt_dist.keys())
        bt_weights = list(bt_dist.values())
        bio = BIOMETRICS.get(cc, BIOMETRICS["US"])
        h_key = "height_m" if person.gender == "male" else "height_f"
        w_key = "weight_m" if person.gender == "male" else "weight_f"
        ins_providers = INSURANCE_PROVIDERS.get(cc, ["National Insurance"])
        person.medical = MedicalRecord(
            blood_type=weighted_choice(rng, bt_items, bt_weights),
            height_cm=bell_curve_int(rng, int(bio[h_key][0]), int(bio[h_key][2]), mean=bio[h_key][1]),
            weight_kg=bell_curve_int(rng, int(bio[w_key][0]), int(bio[w_key][2]), mean=bio[w_key][1]),
            insurance_provider=rng.choice(ins_providers),
        )

        ctx.register(person)
        return person

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "first_name": self.first_name,
            "last_name": self.last_name,
            "full_name": self.full_name,
            "gender": self.gender,
            "age": self.age,
            "date_of_birth": self.date_of_birth,
            "nationality": self.nationality,
            "national_id": self.national_id,
            "marital_status": self.marital_status,
            "children": self.children,
            "email": self.email,
            "phone": self.phone,
            "phone_carrier": self.phone_carrier,
            "username": self.username,
            "address": self.address.to_dict() if self.address else None,
            "bank": self.bank,
            "credit_card_type": self.credit_card_type,
            "salary": self.salary,
            "salary_currency": self.salary_currency,
            "job_title": self.job_title,
            "department": self.department,
            "company_id": self.company_id,
            "education": self.education.to_dict() if self.education else None,
            "vehicle": self.vehicle.to_dict() if self.vehicle else None,
            "medical": self.medical.to_dict() if self.medical else None,
        }


# ============================================================
# Helper functions
# ============================================================

def _format_pattern(pattern: str, rng) -> str:
    return "".join(str(rng.randint(0, 9)) if c == "#" else c for c in pattern)


def _format_postal(pattern: str, rng) -> str:
    result = []
    for c in pattern:
        if c == "#":
            result.append(str(rng.randint(0, 9)))
        elif c == "?":
            result.append(chr(rng.randint(65, 90)))
        else:
            result.append(c)
    return "".join(result)


def _format_id(pattern: str, rng) -> str:
    """Format national ID: # = digit, ? = letter, * = alphanumeric."""
    result = []
    for c in pattern:
        if c == "#":
            result.append(str(rng.randint(0, 9)))
        elif c == "?":
            result.append(chr(rng.randint(65, 90)))
        elif c == "*":
            result.append(rng.choice("ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"))
        else:
            result.append(c)
    return "".join(result)


def _format_plate(country_code: str, rng) -> str:
    """Generate a locale-appropriate license plate."""
    formats = {
        "US": "??? ####",
        "JP": "?? ###-####",
        "DE": "?? ?? ####",
        "BR": "??? #?##",
        "IN": "?? ## ?? ####",
        "GB": "??## ???",
        "FR": "??-###-??",
        "KR": "## ? ####",
        "ES": "#### ???",
        "SA": "??? ####",
        "NG": "???-###??",
    }
    fmt = formats.get(country_code, "??? ####")
    return _format_id(fmt, rng)


def _romanize(text: str) -> str:
    ascii_chars = [c for c in text if ord(c) < 128 and c.isalpha()]
    if ascii_chars:
        return "".join(ascii_chars)
    h = hashlib.md5(text.encode()).hexdigest()[:8]
    return f"user{h}"


_STREET_NAMES = {
    "US": ["Maple", "Oak", "Cedar", "Elm", "Pine", "Main", "Park", "Washington", "Lincoln", "Jefferson"],
    "JP": ["桜", "梅", "松", "竹", "富士", "大和", "瑞穂"],
    "DE": ["Goethe", "Schiller", "Mozart", "Bach", "Beethoven", "Linden", "Berliner", "Münchner", "Rhein"],
    "BR": ["Ipanema", "Copacabana", "Liberdade", "Consolação", "Augusta", "Paulista", "Tiradentes"],
    "IN": ["Mahatma Gandhi", "Nehru", "Tagore", "Subhash", "Patel", "Ambedkar", "Shivaji", "Rajaji", "Tilak"],
    "GB": ["Victoria", "Church", "High", "Station", "Mill", "King", "Queen", "Castle", "Bridge", "Manor"],
    "FR": ["Victor Hugo", "Pasteur", "Voltaire", "Molière", "Liberté", "République", "Gambetta", "Leclerc"],
    "KR": ["세종대", "강남대", "테헤란", "올림픽", "영동대", "압구정", "삼성"],
    "ES": ["Gran Vía", "Castellana", "Alcalá", "Cervantes", "Colón", "Goya", "Serrano", "Velázquez"],
    "SA": ["الملك فهد", "الملك عبدالعزيز", "العروبة", "التحلية", "الأمير سلطان", "الثلاثين"],
    "NG": ["Broad", "Marina", "Ahmadu Bello", "Herbert Macaulay", "Adeola Odeku", "Allen", "Awolowo"],
}

_DISTRICTS = {
    "JP": {"wards": ["中央", "港", "新宿", "渋谷", "千代田", "品川", "目黒", "世田谷"], "districts": ["本町", "元町", "旭町", "栄町", "幸町", "緑町"]},
    "KR": {"wards": ["강남", "서초", "마포", "영등포", "종로", "용산", "송파", "강서"], "districts": ["역삼", "삼성", "논현", "신사", "청담", "대치"]},
    "SA": {"wards": ["العليا", "السلامة", "النزهة", "الروضة", "المروج"], "districts": ["الورود", "الملز", "السليمانية", "الربوة", "المرسلات"]},
    "IN": {"wards": ["Andheri", "Bandra", "Koramangala", "Whitefield", "Salt Lake", "T Nagar", "Jubilee Hills"], "districts": ["Sector 5", "Block A", "Phase 2", "Extension", "Layout", "Nagar"]},
}


def _get_street_names(country_code: str) -> list[str]:
    return _STREET_NAMES.get(country_code, _STREET_NAMES["US"])



"""Tests for fabrikate v0.3.0 — contextually coherent mock data."""

import json
from fabrikate import World, Context, Person, Company, Product, Transaction
from fabrikate.locales import get_locale, supported_locales


def test_person_coherence_japan():
    world = World(locale="ja_JP", seed=1)
    person = world.person()
    assert person.phone.startswith("+81")
    assert person.address.country == "日本"
    assert person.bank in get_locale("ja_JP").banks


def test_person_coherence_us():
    world = World(locale="en_US", seed=2)
    person = world.person()
    assert person.phone.startswith("+1")
    assert person.address.country == "United States"


def test_person_coherence_germany():
    world = World(locale="de_DE", seed=3)
    person = world.person()
    assert person.phone.startswith("+49")
    assert person.address.country == "Deutschland"
    assert person.bank in get_locale("de_DE").banks


def test_company_employees_inherit_context():
    world = World(locale="pt_BR", seed=10)
    company = world.company(industry="tech")
    employees = company.hire(5)
    assert len(employees) == 5
    assert company.currency == "BRL"
    for emp in employees:
        assert emp.company_id == company.id
        assert emp.address.country == "Brasil"
        assert emp.phone.startswith("+55")


def test_seed_reproducibility():
    world1 = World(locale="en_US", seed=42)
    world2 = World(locale="en_US", seed=42)
    p1 = world1.person()
    p2 = world2.person()
    assert p1.first_name == p2.first_name
    assert p1.last_name == p2.last_name
    assert p1.phone == p2.phone
    assert p1.email == p2.email


def test_products_industry_appropriate():
    world = World(locale="en_US", seed=5, industry="health")
    product = world.product()
    assert product.currency == "USD"
    assert product.category == "health"


def test_transactions_link_entities():
    world = World(locale="en_US", seed=7)
    alice = world.person()
    company = world.company()
    tx = world.transaction(sender=alice, receiver=company)
    assert tx.sender_id == alice.id
    assert tx.receiver_id == company.id
    assert tx.currency == "USD"


def test_world_dataset_export():
    world = World(locale="de_DE", seed=99)
    world.company(industry="tech").hire(3)
    world.people(2)
    world.products(4)
    world.transactions(10)
    dataset = world.dataset()
    assert len(dataset["companies"]) == 1
    assert len(dataset["people"]) == 5
    assert len(dataset["products"]) == 4
    assert len(dataset["transactions"]) == 10
    json_str = world.to_json()
    parsed = json.loads(json_str)
    assert parsed["metadata"]["locale"] == "de_DE"


def test_context_child():
    ctx = Context(locale="ja_JP", seed=1, industry="tech")
    child = ctx.child(locale="en_US")
    assert child.locale == "en_US"
    assert child.industry == "tech"


def test_gender_override():
    world = World(locale="en_US", seed=1)
    p = world.person(gender="female")
    assert p.gender == "female"


# --- New v0.3.0 tests ---

def test_person_has_new_fields():
    """Person should have all new v0.3.0 fields populated."""
    world = World(locale="en_US", seed=42)
    p = world.person()
    assert p.date_of_birth != ""
    assert p.national_id != ""
    assert p.marital_status in ["single", "married", "engaged", "divorced", "widowed"]
    assert p.children >= 0
    assert p.phone_carrier != ""
    assert p.username != ""
    assert p.credit_card_type != ""
    assert p.salary > 0
    assert p.salary_currency == "USD"
    assert p.job_title != ""
    assert p.department != ""
    assert p.education is not None
    assert p.education.university != ""
    assert p.education.degree != ""
    assert p.medical is not None
    assert p.medical.blood_type != ""
    assert p.medical.height_cm > 0
    assert p.medical.weight_kg > 0
    assert p.medical.insurance_provider != ""


def test_person_india_new_fields():
    """Indian person should have India-specific new fields."""
    world = World(locale="hi_IN", seed=42)
    p = world.person()
    assert p.nationality == "India"
    assert p.phone_carrier in ["Jio", "Airtel", "Vi (Vodafone Idea)", "BSNL", "MTNL"]
    assert p.salary_currency == "INR"
    assert p.salary >= 300000  # minimum Indian salary in data


def test_company_has_new_fields():
    """Company should have tax_id, revenue, website, departments."""
    world = World(locale="en_US", seed=42)
    c = world.company()
    assert c.tax_id != ""
    assert c.annual_revenue > 0
    assert c.website.startswith("https://")
    assert len(c.departments) >= 2


def test_transaction_has_new_fields():
    """Transaction should have payment_method, tax, invoice."""
    world = World(locale="hi_IN", seed=42)
    tx = world.transaction()
    assert tx.payment_method != ""
    assert tx.tax_amount >= 0
    assert tx.total_amount != 0
    assert tx.invoice_number.startswith("INV-")


def test_transaction_payment_method_locale():
    """Indian transactions should use Indian payment methods."""
    world = World(locale="hi_IN", seed=42)
    methods = set()
    for _ in range(50):
        tx = world.transaction()
        methods.add(tx.payment_method)
    assert "UPI" in methods  # UPI should appear for India


def test_vehicle_generation():
    """Vehicles should be locale-appropriate."""
    world = World(locale="hi_IN", seed=42)
    people_with_cars = [p for p in world.people(20) if p.vehicle is not None]
    assert len(people_with_cars) > 0
    brands = {p.vehicle.brand for p in people_with_cars}
    assert "Maruti Suzuki" in brands or "Hyundai" in brands or "Tata" in brands


def test_all_locales_generate_without_error():
    """Every supported locale should generate a full dataset without crashing."""
    for loc in supported_locales():
        world = World(locale=loc, seed=100)
        company = world.company()
        company.hire(3)
        world.products(2, company=company)
        world.transactions(5)
        data = world.dataset()
        assert len(data["people"]) == 3, f"{loc}: expected 3 people"
        assert len(data["companies"]) == 1, f"{loc}: expected 1 company"
        assert len(data["products"]) == 2, f"{loc}: expected 2 products"
        assert len(data["transactions"]) == 5, f"{loc}: expected 5 transactions"


def test_person_to_dict_complete():
    """to_dict should include all new fields."""
    world = World(locale="en_US", seed=1)
    d = world.person().to_dict()
    required_keys = [
        "id", "first_name", "last_name", "full_name", "gender", "age",
        "date_of_birth", "nationality", "national_id", "marital_status",
        "children", "email", "phone", "phone_carrier", "username", "address",
        "bank", "credit_card_type", "salary", "salary_currency", "job_title",
        "department", "education", "vehicle", "medical",
    ]
    for key in required_keys:
        assert key in d, f"Missing key: {key}"


if __name__ == "__main__":
    import sys
    tests = [v for k, v in globals().items() if k.startswith("test_")]
    passed = 0
    for test in tests:
        try:
            test()
            print(f"  ✓ {test.__name__}")
            passed += 1
        except Exception as e:
            print(f"  ✗ {test.__name__}: {e}")
    print(f"\n{passed}/{len(tests)} tests passed")
    sys.exit(0 if passed == len(tests) else 1)



"""
Transaction generator — creates financial transactions between
entities that make sense: correct currency, realistic amounts,
proper date ranges, and references to existing people/companies.
"""

from __future__ import annotations

import uuid
from dataclasses import dataclass, field
from datetime import datetime, timedelta
from typing import Optional

from fabrikate.context import Context
from fabrikate.locales import get_locale
from fabrikate.distributions import log_normal_float, weighted_choice
from fabrikate.reference_data import PAYMENT_METHODS


TRANSACTION_TYPES = ["purchase", "refund", "subscription", "transfer", "payment", "invoice"]
STATUSES = ["completed", "pending", "failed", "cancelled"]

# Tax rates by country (approximate standard rates)
TAX_RATES: dict[str, float] = {
    "US": 0.08, "JP": 0.10, "DE": 0.19, "BR": 0.17, "IN": 0.18,
    "GB": 0.20, "FR": 0.20, "KR": 0.10, "ES": 0.21, "SA": 0.15, "NG": 0.075,
}


@dataclass
class Transaction:
    """A financial transaction with payment method, tax, and invoice."""

    id: str = ""
    type: str = ""
    amount: float = 0.0
    tax_amount: float = 0.0
    total_amount: float = 0.0
    currency: str = ""
    status: str = ""
    date: str = ""
    payment_method: str = ""
    invoice_number: str = ""
    sender_id: Optional[str] = None
    receiver_id: Optional[str] = None
    product_id: Optional[str] = None
    description: str = ""
    _ctx: Optional[Context] = field(default=None, repr=False)

    @classmethod
    def generate(
        cls,
        ctx: Context,
        sender_id: Optional[str] = None,
        receiver_id: Optional[str] = None,
        product_id: Optional[str] = None,
        **overrides,
    ) -> Transaction:
        locale = get_locale(ctx.locale)
        rng = ctx._rng
        cc = ctx.country_code

        tx = cls(_ctx=ctx)
        tx.id = overrides.get("id", str(uuid.UUID(int=rng.getrandbits(128))))

        tx.type = overrides.get("type", rng.choice(TRANSACTION_TYPES))
        tx.status = overrides.get(
            "status",
            rng.choices(STATUSES, weights=[70, 15, 10, 5])[0],
        )

        # Amount — log-normal (many small, few large)
        if tx.type == "subscription":
            tx.amount = round(log_normal_float(rng, 15.0, sigma=0.7, low=4.99, high=99.99), 2)
        elif tx.type == "refund":
            tx.amount = -round(log_normal_float(rng, 50.0, sigma=0.8, low=5.0, high=500.0), 2)
        else:
            tx.amount = round(log_normal_float(rng, 120.0, sigma=1.0, low=1.0, high=10000.0), 2)
        tx.amount = overrides.get("amount", tx.amount)

        # Tax
        tax_rate = TAX_RATES.get(cc, 0.10)
        tx.tax_amount = overrides.get("tax_amount", round(abs(tx.amount) * tax_rate, 2))
        tx.total_amount = round(tx.amount + (tx.tax_amount if tx.amount > 0 else -tx.tax_amount), 2)

        tx.currency = locale.currency
        tx.sender_id = sender_id
        tx.receiver_id = receiver_id
        tx.product_id = product_id

        # Payment method — locale-weighted
        pm_data = PAYMENT_METHODS.get(cc, PAYMENT_METHODS["US"])
        pm_items = [p[0] for p in pm_data]
        pm_weights = [p[1] for p in pm_data]
        tx.payment_method = overrides.get("payment_method", weighted_choice(rng, pm_items, pm_weights))

        # Invoice number
        tx.invoice_number = overrides.get(
            "invoice_number",
            f"INV-{ctx.year}-{rng.randint(10000, 99999)}",
        )

        # Date
        base = datetime(ctx.year, 1, 1)
        offset = timedelta(days=rng.randint(0, 364))
        tx.date = overrides.get("date", (base + offset).strftime("%Y-%m-%d"))

        tx.description = overrides.get(
            "description",
            f"{tx.type.title()} — {locale.currency_symbol}{abs(tx.amount):.2f}",
        )

        ctx.register(tx)
        return tx

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "type": self.type,
            "amount": self.amount,
            "tax_amount": self.tax_amount,
            "total_amount": self.total_amount,
            "currency": self.currency,
            "status": self.status,
            "date": self.date,
            "payment_method": self.payment_method,
            "invoice_number": self.invoice_number,
            "sender_id": self.sender_id,
            "receiver_id": self.receiver_id,
            "product_id": self.product_id,
            "description": self.description,
        }



"""
Locale-specific data for coherent mock generation.

Each locale contains names, addresses, phone formats, banks, and other
culturally-appropriate data. This is what makes fabrikate different from
random generators — everything stays consistent within a locale.

v0.2.0: Expanded to ~100 names per gender per locale (200+ names total).
"""

from __future__ import annotations

from dataclasses import dataclass, field
from importlib import import_module


@dataclass
class LocaleData:
    first_names_male: list[str] = field(default_factory=list)
    first_names_female: list[str] = field(default_factory=list)
    last_names: list[str] = field(default_factory=list)
    name_order: str = "given_family"  # or "family_given"
    cities: list[str] = field(default_factory=list)
    street_formats: list[str] = field(default_factory=list)
    postal_code_format: str = "#####"
    phone_format: str = "+1 (###) ###-####"
    currency: str = "USD"
    currency_symbol: str = "$"
    banks: list[str] = field(default_factory=list)
    email_domains: list[str] = field(default_factory=list)
    company_suffixes: list[str] = field(default_factory=list)
    country_name: str = "United States"


def _load_locale(module_name: str) -> LocaleData:
    """Load a LocaleData from a data module."""
    mod = import_module(f"fabrikate.locales.data.{module_name}")
    return LocaleData(
        first_names_male=mod.FIRST_NAMES_MALE,
        first_names_female=mod.FIRST_NAMES_FEMALE,
        last_names=mod.LAST_NAMES,
        name_order=getattr(mod, "NAME_ORDER", "given_family"),
        cities=mod.CITIES,
        street_formats=mod.STREET_FORMATS,
        postal_code_format=mod.POSTAL_CODE_FORMAT,
        phone_format=mod.PHONE_FORMAT,
        currency=mod.CURRENCY,
        currency_symbol=mod.CURRENCY_SYMBOL,
        banks=mod.BANKS,
        email_domains=mod.EMAIL_DOMAINS,
        company_suffixes=mod.COMPANY_SUFFIXES,
        country_name=mod.COUNTRY_NAME,
    )


# Supported locale codes mapped to their data module names
_LOCALE_MODULES = {
    "en_US": "en_US",
    "ja_JP": "ja_JP",
    "de_DE": "de_DE",
    "pt_BR": "pt_BR",
    "hi_IN": "hi_IN",
    "en_GB": "en_GB",
    "fr_FR": "fr_FR",
    "ko_KR": "ko_KR",
    "es_ES": "es_ES",
    "ar_SA": "ar_SA",
    "en_NG": "en_NG",
}

# Cache loaded locales
_cache: dict[str, LocaleData] = {}


def get_locale(code: str) -> LocaleData:
    """Get locale data, falling back to en_US for unknown locales."""
    if code in _cache:
        return _cache[code]

    if code in _LOCALE_MODULES:
        locale = _load_locale(_LOCALE_MODULES[code])
        _cache[code] = locale
        return locale

    # Try matching just the language part
    lang = code.split("_")[0] if "_" in code else code
    for key, mod_name in _LOCALE_MODULES.items():
        if key.startswith(lang):
            locale = _load_locale(mod_name)
            _cache[code] = locale
            return locale

    # Fallback
    return get_locale("en_US")


def supported_locales() -> list[str]:
    """Return list of all supported locale codes."""
    return list(_LOCALE_MODULES.keys())



"""Germany locale data — de_DE"""

FIRST_NAMES_MALE = [
    "Lukas", "Leon", "Finn", "Elias", "Jonas", "Ben", "Noah", "Paul", "Maximilian", "Felix",
    "Alexander", "Moritz", "Tim", "Julian", "Erik", "Niklas", "Luca", "Jan", "Sebastian",
    "Tobias", "David", "Florian", "Marcel", "Dennis", "Christian", "Philipp", "Patrick",
    "Stefan", "Martin", "Thomas", "Andreas", "Markus", "Oliver", "Daniel", "Matthias",
    "Frank", "Peter", "Klaus", "Hans", "Werner", "Heinz", "Karl", "Friedrich", "Heinrich",
    "Gerhard", "Wolfgang", "Manfred", "Jürgen", "Günter", "Dieter", "Rainer", "Bernd",
    "Uwe", "Ralf", "Holger", "Torsten", "Sven", "Kai", "Dirk", "Ingo", "Lars", "Jens",
    "Marc", "Christoph", "Simon", "Jakob", "Emil", "Oskar", "Theo", "Anton", "Leo", "Mats",
    "Lennart", "Hannes", "Jannik", "Timo", "Nico", "Robin", "Fabian", "Dominik", "Kevin",
    "Mario", "Manuel", "Robert", "Sascha", "Hendrik", "Bastian", "Thorsten", "Volker",
    "Norbert", "Helmut", "Lothar", "Siegfried", "Rudolf", "Erich", "Otto", "Fritz",
    "Gustav", "Albert", "Walter", "Georg", "Hermann",
]

FIRST_NAMES_FEMALE = [
    "Emma", "Mia", "Hannah", "Sophia", "Lina", "Marie", "Lea", "Clara", "Anna", "Emilia",
    "Laura", "Johanna", "Lena", "Charlotte", "Amelie", "Luisa", "Sophie", "Leonie", "Nele",
    "Ella", "Ida", "Frieda", "Mila", "Greta", "Maja", "Julia", "Lisa", "Sarah", "Katharina",
    "Franziska", "Lara", "Jana", "Vanessa", "Nicole", "Sabine", "Andrea", "Claudia",
    "Stefanie", "Sandra", "Birgit", "Monika", "Petra", "Heike", "Karin", "Ursula", "Helga",
    "Ingrid", "Renate", "Christa", "Gertrud", "Elfriede", "Irmgard", "Edith", "Hildegard",
    "Brigitte", "Gisela", "Elisabeth", "Ilse", "Erika", "Marianne", "Anneliese", "Waltraud",
    "Hannelore", "Margarete", "Ruth", "Annika", "Antonia", "Pauline", "Victoria", "Helena",
    "Theresa", "Carolin", "Melanie", "Nadine", "Tanja", "Anja", "Diana", "Silke", "Daniela",
    "Katrin", "Susanne", "Marion", "Martina", "Bettina", "Sonja", "Yvonne", "Simone",
    "Angelika", "Gabriele", "Christine", "Barbara", "Eva", "Maria", "Rosemarie", "Lieselotte",
    "Annette", "Beate", "Cornelia", "Doris", "Elke",
]

LAST_NAMES = [
    "Müller", "Schmidt", "Schneider", "Fischer", "Weber", "Meyer", "Wagner", "Becker",
    "Schulz", "Hoffmann", "Koch", "Richter", "Wolf", "Schröder", "Neumann", "Schwarz",
    "Zimmermann", "Braun", "Krüger", "Hofmann", "Hartmann", "Lange", "Schmitt", "Werner",
    "Schmitz", "Krause", "Meier", "Lehmann", "Schmid", "Schulze", "Maier", "Köhler",
    "Herrmann", "König", "Walter", "Mayer", "Huber", "Kaiser", "Fuchs", "Peters",
    "Lang", "Scholz", "Möller", "Weiß", "Jung", "Hahn", "Schubert", "Vogel", "Friedrich",
    "Keller", "Günther", "Frank", "Berger", "Winkler", "Roth", "Beck", "Lorenz", "Baumann",
    "Franke", "Albrecht", "Schuster", "Simon", "Ludwig", "Böhm", "Winter", "Kraus", "Martin",
    "Schumacher", "Krämer", "Vogt", "Stein", "Jäger", "Otto", "Sommer", "Groß", "Seidel",
    "Heinrich", "Brandt", "Haas", "Schreiber", "Graf", "Schultz", "Dietrich", "Ziegler",
    "Kuhn", "Kühn", "Pohl", "Engel", "Horn", "Busch", "Bergmann", "Thomas", "Voigt",
    "Sauer", "Arnold", "Wolff", "Pfeiffer", "Brinkmann", "Kraft", "Marx", "Ritter", "Stark",
]

CITIES = [
    "Berlin", "Hamburg", "München", "Köln", "Frankfurt", "Stuttgart", "Düsseldorf", "Leipzig",
    "Dortmund", "Essen", "Bremen", "Dresden", "Hannover", "Nürnberg", "Duisburg", "Bochum",
    "Wuppertal", "Bielefeld", "Bonn", "Münster", "Mannheim", "Karlsruhe", "Augsburg",
    "Wiesbaden", "Mönchengladbach", "Gelsenkirchen", "Aachen", "Braunschweig", "Kiel",
    "Chemnitz", "Halle", "Magdeburg", "Freiburg", "Lübeck", "Erfurt",
]

STREET_FORMATS = [
    "{name}straße {number}", "{name}weg {number}", "{name}allee {number}",
    "{name}platz {number}", "{name}ring {number}", "{name}damm {number}",
]

POSTAL_CODE_FORMAT = "#####"
PHONE_FORMAT = "+49 ### #######"
CURRENCY = "EUR"
CURRENCY_SYMBOL = "€"

BANKS = [
    "Deutsche Bank", "Commerzbank", "DZ Bank", "KfW", "Sparkasse", "Volksbank",
    "Postbank", "HypoVereinsbank", "ING-DiBa", "Targobank", "Landesbank", "N26",
]

EMAIL_DOMAINS = ["gmail.com", "web.de", "gmx.de", "t-online.de", "outlook.de", "freenet.de"]
COMPANY_SUFFIXES = ["GmbH", "AG", "e.V.", "KG", "OHG", "SE", "GmbH & Co. KG"]
COUNTRY_NAME = "Deutschland"
NAME_ORDER = "given_family"
